@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 2002-2021 The Octave Project Developers
+// Copyright (C) 2002-2023 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -28,6 +28,7 @@
 #endif
 
 #include "file-stat.h"
+#include "lo-sysdep.h"
 #include "oct-env.h"
 #include "oct-time.h"
 
@@ -38,9 +39,18 @@
 #include "errwarn.h"
 
 #if defined (HAVE_MAGICK)
+#  include <Magick++.h>
+#  include <clocale>
+// FIXME: The following using declaration may be needed to build with
+// ImageMagick.  It doesn't appear to be needed for GraphicsMagick but
+// it also doesn't seem to cause trouble.  A configure test would be
+// helpful.
+using Magick::Quantum;
+#endif
 
-#include <Magick++.h>
-#include <clocale>
+OCTAVE_BEGIN_NAMESPACE(octave)
+
+#if defined (HAVE_MAGICK)
 
 // In theory, it should be enough to check the class:
 // Magick::ClassType
@@ -140,17 +150,17 @@ get_depth (Magick::Image& img)
 // width 1.  In those cases, the type will come as scalar instead of range
 // since that's the behavior of the colon operator (1:1:1 will be a scalar,
 // not a range).
-static Range
+static range<double>
 get_region_range (const octave_value& region)
 {
-  Range output;
+  range<double> output;
 
   if (region.is_range ())
     output = region.range_value ();
   else if (region.is_scalar_type ())
     {
       double value = region.scalar_value ();
-      output = Range (value, value);
+      output = range<double> (value, value);
     }
   else if (region.is_matrix_type ())
     {
@@ -158,7 +168,7 @@ get_region_range (const octave_value& region)
       double base = array(0);
       double limit = array(array.numel () - 1);
       double incr = array(1) - base;
-      output = Range (base, limit, incr);
+      output = range<double> (base, incr, limit);
     }
   else
     error ("__magick_read__: unknown datatype for Region option");
@@ -180,8 +190,8 @@ public:
 
     // Subtract 1 to account for 0 indexing.
 
-    const Range rows = get_region_range (pixel_region (0));
-    const Range cols = get_region_range (pixel_region (1));
+    const range<double> rows = get_region_range (pixel_region (0));
+    const range<double> cols = get_region_range (pixel_region (1));
 
     m_row_start = rows.base () - 1;
     m_col_start = cols.base () - 1;
@@ -191,8 +201,8 @@ public:
     m_row_cache = m_row_end - m_row_start + 1;
     m_col_cache = m_col_end - m_col_start + 1;
 
-    m_row_shift = m_col_cache * rows.inc ();
-    m_col_shift = m_col_cache * (m_row_cache + rows.inc () - 1) - cols.inc ();
+    m_row_shift = m_col_cache * rows.increment ();
+    m_col_shift = m_col_cache * (m_row_cache + rows.increment () - 1) - cols.increment ();
 
     m_row_out = rows.numel ();
     m_col_out = cols.numel ();
@@ -263,9 +273,9 @@ read_maps (Magick::Image& img)
   for (octave_idx_type i = 0; i < mapsize; i++)
     {
       const Magick::ColorRGB c = img.colorMap (i);
-      cmap(i,0) = c.red   ();
-      cmap(i,1) = c.green ();
-      cmap(i,2) = c.blue  ();
+      cmap(i, 0) = c.red   ();
+      cmap(i, 1) = c.green ();
+      cmap(i, 2) = c.blue  ();
       amap(i)   = c.alpha ();
     }
   octave_value_list maps;
@@ -349,7 +359,7 @@ read_indexed_images (const std::vector<Magick::Image>& imvec,
       if (imvec[def_elem].matte () && nargout >= 3)
         {
           const Matrix amap = maps(1).matrix_value ();
-          const double *amap_fvec = amap.fortran_vec ();
+          const double *amap_fvec = amap.data ();
 
           NDArray alpha (dim_vector (nRows, nCols, 1, nFrames));
           double *alpha_fvec = alpha.fortran_vec ();
@@ -523,7 +533,7 @@ read_images (std::vector<Magick::Image>& imvec,
 
             const Magick::PixelPacket *pix
               = imvec[frameidx(frame)].getConstPixels (col_start, row_start,
-                                                       col_cache, row_cache);
+                  col_cache, row_cache);
 
             for (octave_idx_type col = 0; col < nCols; col++)
               {
@@ -552,7 +562,7 @@ read_images (std::vector<Magick::Image>& imvec,
 
             const Magick::PixelPacket *pix
               = imvec[frameidx(frame)].getConstPixels (col_start, row_start,
-                                                       col_cache, row_cache);
+                  col_cache, row_cache);
 
             for (octave_idx_type col = 0; col < nCols; col++)
               {
@@ -583,7 +593,7 @@ read_images (std::vector<Magick::Image>& imvec,
 
             const Magick::PixelPacket *pix
               = imvec[frameidx(frame)].getConstPixels (col_start, row_start,
-                                                       col_cache, row_cache);
+                  col_cache, row_cache);
 
             octave_idx_type idx = 0;
             P *rbuf = img_fvec;
@@ -626,7 +636,7 @@ read_images (std::vector<Magick::Image>& imvec,
 
             const Magick::PixelPacket *pix
               = imvec[frameidx(frame)].getConstPixels (col_start, row_start,
-                                                       col_cache, row_cache);
+                  col_cache, row_cache);
 
             octave_idx_type idx = 0;
             P *rbuf = img_fvec;
@@ -664,7 +674,7 @@ read_images (std::vector<Magick::Image>& imvec,
 
             const Magick::PixelPacket *pix
               = imvec[frameidx(frame)].getConstPixels (col_start, row_start,
-                                                       col_cache, row_cache);
+                  col_cache, row_cache);
 
             octave_idx_type idx = 0;
             P *cbuf = img_fvec;
@@ -709,7 +719,7 @@ read_images (std::vector<Magick::Image>& imvec,
 
             const Magick::PixelPacket *pix
               = imvec[frameidx(frame)].getConstPixels (col_start, row_start,
-                                                       col_cache, row_cache);
+                  col_cache, row_cache);
             // Note that for CMYKColorspace + matte (CMYKA), the opacity is
             // stored in the associated IndexPacket.
             const Magick::IndexPacket *apix
@@ -754,15 +764,21 @@ read_images (std::vector<Magick::Image>& imvec,
 void static
 read_file (const std::string& filename, std::vector<Magick::Image>& imvec)
 {
+  // FIXME: We need this on Windows because GraphicsMagick uses the ANSI API
+  // to open files on disc.  In contrast, the API of ImageMagick uses UTF-8
+  // encoded strings.  Should we somehow detect which is used on runtime and
+  // pass the file names accordingly? (See also bug #58493.)
+  std::string ascii_fname = sys::get_ASCII_filename (filename, true);
+
   try
     {
-      Magick::readImages (&imvec, filename);
+      Magick::readImages (&imvec, ascii_fname);
     }
-  catch (Magick::Warning& w)
+  catch (const Magick::Warning& w)
     {
       warning ("Magick++ warning: %s", w.what ());
     }
-  catch (Magick::Exception& e)
+  catch (const Magick::Exception& e)
     {
       error ("Magick++ exception: %s", e.what ());
     }
@@ -781,7 +797,7 @@ maybe_initialize_magick (void)
       const std::string locale = (static_locale ? static_locale : "");
 
       const std::string program_name
-        = octave::sys::env::get_program_invocation_name ();
+        = sys::env::get_program_invocation_name ();
       Magick::InitializeMagick (program_name.c_str ());
 
       // Restore locale from before GraphicsMagick initialisation
@@ -898,13 +914,13 @@ Use @code{imread} instead.
     {
       if (depth <= 1)
         output = read_indexed_images<boolNDArray>   (imvec, frameidx,
-                                                     nargout, options);
+                 nargout, options);
       else if (depth <= 8)
         output = read_indexed_images<uint8NDArray>  (imvec, frameidx,
-                                                     nargout, options);
+                 nargout, options);
       else if (depth <= 16)
         output = read_indexed_images<uint16NDArray> (imvec, frameidx,
-                                                     nargout, options);
+                 nargout, options);
       else
         error ("imread: indexed images with depths greater than 16-bit are not supported");
     }
@@ -951,7 +967,7 @@ img_float2uint (const T& img)
   uint32NDArray out (img.dims ());
 
   octave_uint32 *out_fvec = out.fortran_vec ();
-  const P       *img_fvec = img.fortran_vec ();
+  const P       *img_fvec = img.data ();
 
   const octave_uint32 max = octave_uint32::max ();
   const octave_idx_type numel = img.numel ();
@@ -1021,7 +1037,7 @@ encode_indexed_images (std::vector<Magick::Image>& imvec,
   // even if it's always the same.  We can least get a vector for the Colors.
   std::vector<Magick::ColorRGB> colormap;
   {
-    const double *cmap_fvec = cmap.fortran_vec ();
+    const double *cmap_fvec = cmap.data ();
     const octave_idx_type G_offset = cmap_size;
     const octave_idx_type B_offset = cmap_size * 2;
     for (octave_idx_type map_idx = 0; map_idx < cmap_size; map_idx++)
@@ -1035,8 +1051,8 @@ encode_indexed_images (std::vector<Magick::Image>& imvec,
       octave_quit ();
 
       Magick::Image m_img = init_enconde_image (nCols, nRows, bitdepth,
-                                                Magick::PaletteType,
-                                                Magick::PseudoClass);
+                            Magick::PaletteType,
+                            Magick::PseudoClass);
 
       // Insert colormap.
       m_img.colorMapSize (cmap_size);
@@ -1054,7 +1070,7 @@ encode_indexed_images (std::vector<Magick::Image>& imvec,
       // over the order of that colormap.  And that's why we set both.
       Magick::PixelPacket *pix = m_img.getPixels (0, 0, nCols, nRows);
       Magick::IndexPacket *ind = m_img.getIndexes ();
-      const P *img_fvec        = img.fortran_vec ();
+      const P *img_fvec        = img.data ();
 
       octave_idx_type GM_idx = 0;
       for (octave_idx_type column = 0; column < nCols; column++)
@@ -1085,7 +1101,7 @@ encode_bool_image (std::vector<Magick::Image>& imvec, const boolNDArray& img)
   // The initialized image will be black, this is for the other pixels
   const Magick::Color white ("white");
 
-  const bool *img_fvec = img.fortran_vec ();
+  const bool *img_fvec = img.data ();
   octave_idx_type img_idx = 0;
   for (octave_idx_type frame = 0; frame < nFrames; frame++)
     {
@@ -1096,8 +1112,8 @@ encode_bool_image (std::vector<Magick::Image>& imvec, const boolNDArray& img)
       // However, this will still work fine and a binary image will be
       // saved because we are setting the bitdepth to 1.
       Magick::Image m_img = init_enconde_image (nCols, nRows, 1,
-                                                Magick::GrayscaleType,
-                                                Magick::DirectClass);
+                            Magick::GrayscaleType,
+                            Magick::DirectClass);
 
       Magick::PixelPacket *pix = m_img.getPixels (0, 0, nCols, nRows);
       octave_idx_type GM_idx = 0;
@@ -1174,8 +1190,8 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
   const double divisor = static_cast<double> ((uint64_t (1) << bitdepth) - 1)
                          / MaxRGB;
 
-  const P *img_fvec = img.fortran_vec ();
-  const P *a_fvec   = alpha.fortran_vec ();
+  const P *img_fvec = img.data ();
+  const P *a_fvec   = alpha.data ();
   switch (type)
     {
     case Magick::GrayscaleType:
@@ -1185,8 +1201,8 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
             octave_quit ();
 
             Magick::Image m_img = init_enconde_image (nCols, nRows, bitdepth,
-                                                      type,
-                                                      Magick::DirectClass);
+                                  type,
+                                  Magick::DirectClass);
 
             Magick::PixelPacket *pix = m_img.getPixels (0, 0, nCols, nRows);
             octave_idx_type GM_idx = 0;
@@ -1194,7 +1210,7 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
               {
                 for (octave_idx_type row = 0; row < nRows; row++)
                   {
-                    const double grey = octave::math::round (double (*img_fvec) / divisor);
+                    const double grey = math::round (double (*img_fvec) / divisor);
                     Magick::Color c (grey, grey, grey);
                     pix[GM_idx] = c;
                     img_fvec++;
@@ -1216,8 +1232,8 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
             octave_quit ();
 
             Magick::Image m_img = init_enconde_image (nCols, nRows, bitdepth,
-                                                      type,
-                                                      Magick::DirectClass);
+                                  type,
+                                  Magick::DirectClass);
 
             Magick::PixelPacket *pix = m_img.getPixels (0, 0, nCols, nRows);
             octave_idx_type GM_idx = 0;
@@ -1225,9 +1241,9 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
               {
                 for (octave_idx_type row = 0; row < nRows; row++)
                   {
-                    double grey = octave::math::round (double (*img_fvec) / divisor);
+                    double grey = math::round (double (*img_fvec) / divisor);
                     Magick::Color c (grey, grey, grey,
-                                     MaxRGB - octave::math::round (double (*a_fvec) / divisor));
+                                     MaxRGB - math::round (double (*a_fvec) / divisor));
                     pix[GM_idx] = c;
                     img_fvec++;
                     a_fvec++;
@@ -1244,7 +1260,7 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
 
     case Magick::TrueColorType:
       {
-        // The fortran_vec offset for the green and blue channels
+        // The data offset for the green and blue channels
         const octave_idx_type G_offset = nCols * nRows;
         const octave_idx_type B_offset = nCols * nRows * 2;
         for (octave_idx_type frame = 0; frame < nFrames; frame++)
@@ -1252,8 +1268,8 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
             octave_quit ();
 
             Magick::Image m_img = init_enconde_image (nCols, nRows, bitdepth,
-                                                      type,
-                                                      Magick::DirectClass);
+                                  type,
+                                  Magick::DirectClass);
 
             Magick::PixelPacket *pix = m_img.getPixels (0, 0, nCols, nRows);
             octave_idx_type GM_idx = 0;
@@ -1261,9 +1277,9 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
               {
                 for (octave_idx_type row = 0; row < nRows; row++)
                   {
-                    Magick::Color c (octave::math::round (double (*img_fvec)          / divisor),
-                                     octave::math::round (double (img_fvec[G_offset]) / divisor),
-                                     octave::math::round (double (img_fvec[B_offset]) / divisor));
+                    Magick::Color c (math::round (double (*img_fvec)          / divisor),
+                                     math::round (double (img_fvec[G_offset]) / divisor),
+                                     math::round (double (img_fvec[B_offset]) / divisor));
                     pix[GM_idx] = c;
                     img_fvec++;
                     GM_idx += nCols;
@@ -1280,7 +1296,7 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
 
     case Magick::TrueColorMatteType:
       {
-        // The fortran_vec offset for the green and blue channels
+        // The data offset for the green and blue channels
         const octave_idx_type G_offset = nCols * nRows;
         const octave_idx_type B_offset = nCols * nRows * 2;
         for (octave_idx_type frame = 0; frame < nFrames; frame++)
@@ -1288,8 +1304,8 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
             octave_quit ();
 
             Magick::Image m_img = init_enconde_image (nCols, nRows, bitdepth,
-                                                      type,
-                                                      Magick::DirectClass);
+                                  type,
+                                  Magick::DirectClass);
 
             Magick::PixelPacket *pix = m_img.getPixels (0, 0, nCols, nRows);
             octave_idx_type GM_idx = 0;
@@ -1297,10 +1313,10 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
               {
                 for (octave_idx_type row = 0; row < nRows; row++)
                   {
-                    Magick::Color c (octave::math::round (double (*img_fvec)          / divisor),
-                                     octave::math::round (double (img_fvec[G_offset]) / divisor),
-                                     octave::math::round (double (img_fvec[B_offset]) / divisor),
-                                     MaxRGB - octave::math::round (double (*a_fvec) / divisor));
+                    Magick::Color c (math::round (double (*img_fvec)          / divisor),
+                                     math::round (double (img_fvec[G_offset]) / divisor),
+                                     math::round (double (img_fvec[B_offset]) / divisor),
+                                     MaxRGB - math::round (double (*a_fvec) / divisor));
                     pix[GM_idx] = c;
                     img_fvec++;
                     a_fvec++;
@@ -1318,7 +1334,7 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
 
     case Magick::ColorSeparationType:
       {
-        // The fortran_vec offset for the Magenta, Yellow, and blacK channels
+        // The data offset for the Magenta, Yellow, and blacK channels
         const octave_idx_type M_offset = nCols * nRows;
         const octave_idx_type Y_offset = nCols * nRows * 2;
         const octave_idx_type K_offset = nCols * nRows * 3;
@@ -1327,8 +1343,8 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
             octave_quit ();
 
             Magick::Image m_img = init_enconde_image (nCols, nRows, bitdepth,
-                                                      type,
-                                                      Magick::DirectClass);
+                                  type,
+                                  Magick::DirectClass);
 
             Magick::PixelPacket *pix = m_img.getPixels (0, 0, nCols, nRows);
             octave_idx_type GM_idx = 0;
@@ -1336,10 +1352,10 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
               {
                 for (octave_idx_type row = 0; row < nRows; row++)
                   {
-                    Magick::Color c (octave::math::round (double (*img_fvec)          / divisor),
-                                     octave::math::round (double (img_fvec[M_offset]) / divisor),
-                                     octave::math::round (double (img_fvec[Y_offset]) / divisor),
-                                     octave::math::round (double (img_fvec[K_offset]) / divisor));
+                    Magick::Color c (math::round (double (*img_fvec)          / divisor),
+                                     math::round (double (img_fvec[M_offset]) / divisor),
+                                     math::round (double (img_fvec[Y_offset]) / divisor),
+                                     math::round (double (img_fvec[K_offset]) / divisor));
                     pix[GM_idx] = c;
                     img_fvec++;
                     GM_idx += nCols;
@@ -1356,7 +1372,7 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
 
     case Magick::ColorSeparationMatteType:
       {
-        // The fortran_vec offset for the Magenta, Yellow, and blacK channels
+        // The data offset for the Magenta, Yellow, and blacK channels
         const octave_idx_type M_offset = nCols * nRows;
         const octave_idx_type Y_offset = nCols * nRows * 2;
         const octave_idx_type K_offset = nCols * nRows * 3;
@@ -1365,8 +1381,8 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
             octave_quit ();
 
             Magick::Image m_img = init_enconde_image (nCols, nRows, bitdepth,
-                                                      type,
-                                                      Magick::DirectClass);
+                                  type,
+                                  Magick::DirectClass);
 
             Magick::PixelPacket *pix = m_img.getPixels (0, 0, nCols, nRows);
             Magick::IndexPacket *ind = m_img.getIndexes ();
@@ -1375,12 +1391,12 @@ encode_uint_image (std::vector<Magick::Image>& imvec,
               {
                 for (octave_idx_type row = 0; row < nRows; row++)
                   {
-                    Magick::Color c (octave::math::round (double (*img_fvec)          / divisor),
-                                     octave::math::round (double (img_fvec[M_offset]) / divisor),
-                                     octave::math::round (double (img_fvec[Y_offset]) / divisor),
-                                     octave::math::round (double (img_fvec[K_offset]) / divisor));
+                    Magick::Color c (math::round (double (*img_fvec)          / divisor),
+                                     math::round (double (img_fvec[M_offset]) / divisor),
+                                     math::round (double (img_fvec[Y_offset]) / divisor),
+                                     math::round (double (img_fvec[K_offset]) / divisor));
                     pix[GM_idx] = c;
-                    ind[GM_idx] = MaxRGB - octave::math::round (double (*a_fvec) / divisor);
+                    ind[GM_idx] = MaxRGB - math::round (double (*a_fvec) / divisor);
                     img_fvec++;
                     a_fvec++;
                     GM_idx += nCols;
@@ -1454,15 +1470,15 @@ write_file (const std::string& filename,
     {
       Magick::writeImages (imvec.begin (), imvec.end (), ext + ':' + filename);
     }
-  catch (Magick::Warning& w)
+  catch (const Magick::Warning& w)
     {
       warning ("Magick++ warning: %s", w.what ());
     }
-  catch (Magick::ErrorCoder& e)
+  catch (const Magick::ErrorCoder& e)
     {
       warning ("Magick++ coder error: %s", e.what ());
     }
-  catch (Magick::Exception& e)
+  catch (const Magick::Exception& e)
     {
       error ("Magick++ exception: %s", e.what ());
     }
@@ -1525,9 +1541,9 @@ Use @code{imwrite} instead.
           if (img.is_single_type ())
             {
               clip_img   = img_float2uint<FloatNDArray>
-                             (img.float_array_value ());
+                           (img.float_array_value ());
               clip_alpha = img_float2uint<FloatNDArray>
-                             (alpha.float_array_value ());
+                           (alpha.float_array_value ());
             }
           else
             {
@@ -1551,7 +1567,7 @@ Use @code{imwrite} instead.
         encode_indexed_images<uint16NDArray> (imvec, img.uint16_array_value (),
                                               cmap);
       else
-        error ("__magick_write__: indexed image must be uint8, uint16 or float.");
+        error ("__magick_write__: indexed image must be uint8, uint16 or float");
     }
   static std::map<std::string, octave_idx_type> disposal_methods
     = init_reverse_disposal_methods ();
@@ -1573,7 +1589,7 @@ Use @code{imwrite} instead.
   // If writemode is set to append, read the image and append to it.  Even
   // if set to append, make sure that something was read at all.
   const std::string writemode = options.getfield ("writemode").string_value ();
-  if (writemode == "append" && octave::sys::file_stat (filename).exists ())
+  if (writemode == "append" && sys::file_stat (filename).exists ())
     {
       std::vector<Magick::Image> ini_imvec;
       read_file (filename, ini_imvec);
@@ -1661,7 +1677,7 @@ Use @code{imwrite} instead.
 // done in Octave language), and then again for the actual reading.
 DEFUN (__magick_ping__, args, ,
        doc: /* -*- texinfo -*-
-@deftypefn {} {} __magick_ping__ (@var{fname}, @var{idx})
+@deftypefn {} {@var{fmt} =} __magick_ping__ (@var{fname}, @var{idx})
 Ping image information with GraphicsMagick or ImageMagick.
 
 This is a private internal function not intended for direct use.
@@ -1687,15 +1703,22 @@ This is a private internal function not intended for direct use.
   Magick::Image img;
   img.subImage (idx); // start ping from this image (in case of multi-page)
   img.subRange (1);   // ping only one of them
+
+  // FIXME: We need this on Windows because GraphicsMagick uses the ANSI API
+  // to open files on disc.  In contrast, the API of ImageMagick uses UTF-8
+  // encoded strings.  Should we somehow detect which is used on runtime and
+  // pass the file names accordingly? (See also bug #58493.)
+  std::string ascii_fname = sys::get_ASCII_filename (filename, true);
+
   try
     {
-      img.ping (filename);
+      img.ping (ascii_fname);
     }
-  catch (Magick::Warning& w)
+  catch (const Magick::Warning& w)
     {
       warning ("Magick++ warning: %s", w.what ());
     }
-  catch (Magick::Exception& e)
+  catch (const Magick::Exception& e)
     {
       error ("Magick++ exception: %s", e.what ());
     }
@@ -1887,7 +1910,7 @@ fill_exif_floats (octave_scalar_map& map, Magick::Image& img,
 
 DEFUN (__magick_finfo__, args, ,
        doc: /* -*- texinfo -*-
-@deftypefn {} {} __magick_finfo__ (@var{fname})
+@deftypefn {} {@var{infostruct} =} __magick_finfo__ (@var{fname})
 Read image information with GraphicsMagick or ImageMagick.
 
 This is a private internal function not intended for direct use.
@@ -1992,12 +2015,12 @@ Use @code{imfinfo} instead.
   // We can't actually get FormatVersion but even Matlab sometimes can't.
   template_info.setfield ("FormatVersion", octave_value (""));
 
-  const octave::sys::file_stat fs (filename);
+  const sys::file_stat fs (filename);
   if (! fs)
     error ("imfinfo: error reading '%s': %s", filename.c_str (),
            fs.error ().c_str ());
 
-  const octave::sys::localtime mtime (fs.mtime ());
+  const sys::localtime mtime (fs.mtime ());
   const std::string filetime = mtime.strftime ("%e-%b-%Y %H:%M:%S");
   template_info.setfield ("Filename",    octave_value (filename));
   template_info.setfield ("FileModDate", octave_value (filetime));
@@ -2338,7 +2361,7 @@ Use @code{imfinfo} instead.
 
 DEFUN (__magick_formats__, args, ,
        doc: /* -*- texinfo -*-
-@deftypefn {} {} __magick_imformats__ (@var{formats})
+@deftypefn {} {@var{fmt_struct} =} __magick_imformats__ (@var{formats})
 Fill formats info with GraphicsMagick CoderInfo.
 
 @seealso{imfinfo, imformats, imread, imwrite}
@@ -2370,7 +2393,7 @@ Fill formats info with GraphicsMagick CoderInfo.
             fmt.setfield ("write", Matrix ());
           formats.fast_elem_insert (idx, fmt);
         }
-      catch (Magick::Exception& e)
+      catch (const Magick::Exception&)
         {
           // Exception here are missing formats.  So we remove the format
           // from the structure and reduce idx.
@@ -2392,3 +2415,5 @@ Fill formats info with GraphicsMagick CoderInfo.
 ## No test needed for internal helper function.
 %!assert (1)
 */
+
+OCTAVE_END_NAMESPACE(octave)
