@@ -1,25 +1,27 @@
-/*
-
-Copyright (C) 2001-2018 Ben Sapp
-Copyright (C) 2007-2009 John Swensen
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 2001-2023 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -36,7 +38,6 @@ along with Octave; see the file COPYING.  If not, see
 #include "dNDArray.h"
 
 #include "bp-table.h"
-#include "call-stack.h"
 #include "defun.h"
 #include "error.h"
 #include "errwarn.h"
@@ -45,6 +46,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "input.h"
 #include "interpreter-private.h"
 #include "interpreter.h"
+#include "lo-sysdep.h"
 #include "octave-preserve-stream-state.h"
 #include "ov-usr-fcn.h"
 #include "ov.h"
@@ -57,38 +59,33 @@ along with Octave; see the file COPYING.  If not, see
 #include "utils.h"
 #include "variables.h"
 
+OCTAVE_BEGIN_NAMESPACE(octave)
+
 static octave_value
-intmap_to_ov (const octave::bp_table::intmap& line)
+bp_lines_to_ov (const octave::bp_table::bp_lines& lines)
 {
   int idx = 0;
 
-  NDArray retval (dim_vector (1, line.size ()));
+  NDArray retval (dim_vector (1, lines.size ()));
 
-  for (size_t i = 0; i < line.size (); i++)
-    {
-      octave::bp_table::const_intmap_iterator p = line.find (i);
-
-      if (p != line.end ())
-        {
-          int lineno = p->second;
-          retval(idx++) = lineno;
-        }
-    }
+  for (const auto& lineno : lines)
+    retval(idx++) = lineno;
 
   retval.resize (dim_vector (1, idx));
 
   return retval;
 }
 
-DEFUN (dbstop, args, ,
-       doc: /* -*- texinfo -*-
-@deftypefn  {} {} dbstop @var{func}
-@deftypefnx {} {} dbstop @var{func} @var{line}
-@deftypefnx {} {} dbstop @var{func} @var{line1} @var{line2} @dots{}
+DEFMETHOD (dbstop, interp, args, ,
+           doc: /* -*- texinfo -*-
+@deftypefn  {} {} dbstop @var{fcn}
+@deftypefnx {} {} dbstop @var{fcn} @var{line}
+@deftypefnx {} {} dbstop @var{fcn} @var{line1} @var{line2} @dots{}
 @deftypefnx {} {} dbstop @var{line1} @dots{}
-@deftypefnx {} {} dbstop in @var{func}
-@deftypefnx {} {} dbstop in @var{func} at @var{line}
-@deftypefnx {} {} dbstop in @var{func} at @var{line} if "@var{condition}"
+@deftypefnx {} {} dbstop in @var{fcn}
+@deftypefnx {} {} dbstop in @var{fcn} at @var{line}
+@deftypefnx {} {} dbstop in @var{fcn} at @var{line} if "@var{condition}"
+@deftypefnx {} {} dbstop in @var{class} at @var{method}
 @deftypefnx {} {} dbstop if @var{event}
 @deftypefnx {} {} dbstop if @var{event} @var{ID}
 @deftypefnx {} {} dbstop (@var{bp_struct})
@@ -96,16 +93,16 @@ DEFUN (dbstop, args, ,
 
 Set breakpoints for the built-in debugger.
 
-@var{func} is the name of a function on the current @code{path}.  When
-already in debug mode the @var{func} argument can be omitted and the current
+@var{fcn} is the name of a function on the current @code{path}.  When
+already in debug mode the @var{fcn} argument can be omitted and the current
 function will be used.  Breakpoints at subfunctions are set with the scope
 operator @samp{>}.  For example, If @file{file.m} has a subfunction
-@code{func2}, then a breakpoint in @code{func2} can be specified by
-@code{file>func2}.
+@code{fcn2}, then a breakpoint in @code{fcn2} can be specified by
+@code{file>fcn2}.
 
 @var{line} is the line number at which to break.  If @var{line} is not
 specified, it defaults to the first executable line in the file
-@file{func.m}.  Multiple lines can be specified in a single command; when
+@file{fcn.m}.  Multiple lines can be specified in a single command; when
 function syntax is used, the lines may also be passed as a single vector
 argument (@code{[@var{line1}, @var{line2}, @dots{}]}).
 
@@ -114,24 +111,24 @@ context that exists at the breakpoint.  When the breakpoint is encountered,
 @var{condition} will be evaluated, and execution will stop if
 @var{condition} is true.  If @var{condition} cannot be evaluated, for
 example because it refers to an undefined variable, an error will be thrown.
- Expressions with side effects (such as @code{y++ > 1}) will alter
-variables, and should generally be avoided.  Conditions containing quotes
-(@samp{"}, @samp{'}) or comment characters (@samp{#}, @samp{%}) must be
-enclosed in quotes.  (This does not apply to conditions entered from the
-editor's context menu.)  For example:
+Expressions with side effects (such as @code{y++ > 1}) will alter variables,
+and should generally be avoided.  Conditions containing quotes (@samp{"},
+@samp{'}) or comment characters (@samp{#}, @samp{%}) must be enclosed in
+quotes.  (This does not apply to conditions entered from the editor's context
+menu.)  For example:
 
 @example
-dbstop in strread at 209 if 'any (format == "%f")'
+dbstop in axis at 246 if 'any (opt == "x")'
 @end example
 
-The form specifying @var{event} does not cause a specific breakpoint at a
-given function and line number.  Instead it causes debug mode to be entered
-when certain unexpected events are encountered.  Possible values are
+The form specifying @var{event} does not cause a specific breakpoint at a given
+function and line number.  Instead it causes debug mode to be entered when
+certain unexpected events are encountered.  Possible values are
 
 @table @code
 @item error
-Stop when an error is reported.  This is equivalent to specifying
-both @code{debug_on_error (true)} and @code{debug_on_interrupt (true)}.
+Stop when an error is reported.  This is equivalent to specifying both
+@code{debug_on_error (true)} and @code{debug_on_interrupt (true)}.
 
 @item caught error
 Stop when an error is caught by a try-catch block (not yet implemented).
@@ -148,10 +145,9 @@ Stop when a warning is reported.  This is equivalent to specifying
 @end table
 
 The events @code{error}, @code{caught error}, and @code{warning} can all be
-followed by a string specifying an error ID or warning ID@.  If that is
-done, only errors with the specified ID will cause execution to stop.  To
-stop on one of a set of IDs, multiple @code{dbstop} commands must be
-issued.
+followed by a string specifying an error ID or warning ID@.  If that is done,
+only errors with the specified ID will cause execution to stop.  To stop on one
+of a set of IDs, multiple @code{dbstop} commands must be issued.
 
 Breakpoints and events can be removed using the @code{dbclear} command with
 the same syntax.
@@ -161,36 +157,41 @@ the commands @code{bp_state = dbstatus; @dots{}; dbstop (bp_state)}.
 
 The optional output @var{rline} is the real line number where the breakpoint
 was set.  This can differ from the specified line if the line is not
-executable.  For example, if a breakpoint attempted on a blank line then
-Octave will set the real breakpoint at the next executable line.
+executable.  For example, if a breakpoint attempted on a blank line then Octave
+will set the real breakpoint at the next executable line.
 
-When a file is re-parsed, such as when it is modified outside the GUI,
-all breakpoints within the file are cleared.
+When a file is re-parsed, such as when it is modified outside the GUI, all
+breakpoints within the file are cleared.
 
-@seealso{dbclear, dbstatus, dbstep, debug_on_error, debug_on_warning, debug_on_interrupt}
+@seealso{dbclear, dbstatus, dbstep, debug_on_error, debug_on_warning,
+debug_on_interrupt}
 @end deftypefn */)
 {
-  octave::bp_table::intmap retmap;
+  octave::bp_table::bp_lines retmap;
   std::string symbol_name = "";  // stays empty for "dbstop if error" etc
-  octave::bp_table::intmap lines;
+  std::string class_name = "";
+  octave::bp_table::bp_lines lines;
   std::string condition = "";
   octave_value retval;
 
-  octave::bp_table& bptab = octave::__get_bp_table__ ("Fdbstop");
+  octave::tree_evaluator& tw = interp.get_evaluator ();
+
+  octave::bp_table& bptab = tw.get_bp_table ();
 
   if (args.length() >= 1 && ! args(0).isstruct ())
     {
       // explicit function / line / condition
       bptab.parse_dbfunction_params ("dbstop", args, symbol_name,
-                                     lines, condition);
+                                     class_name, lines, condition);
 
       if (lines.size () == 0)
-        lines[0] = 1;
+        lines.insert (1);
 
       if (symbol_name != "")
         {
-          retmap = bptab.add_breakpoint (symbol_name, lines, condition);
-          retval = intmap_to_ov (retmap);
+          retmap = bptab.add_breakpoints_in_function (symbol_name, class_name,
+                   lines, condition);
+          retval = bp_lines_to_ov (retmap);
         }
     }
   else if (args.length () != 1)
@@ -215,10 +216,7 @@ all breakpoints within the file are cleared.
                   && bkpt.cell_value () (0).isstruct ())
                 mv = bkpt.cell_value () (0).map_value ();
               else
-                {
-                  error ("dbstop: invalid 'bkpt' field");
-                  mv = octave_map ();
-                }
+                error ("dbstop: invalid 'bkpt' field");
             }
         }
       if (mv.isempty ())
@@ -228,7 +226,6 @@ all breakpoints within the file are cleared.
       else if (! mv.isfield ("name") || ! mv.isfield ("line"))
         {
           error ("dbstop: Cell array must contain fields 'name' and 'line'");
-          retval = octave_value (0);
         }
       else
         {
@@ -239,41 +236,49 @@ all breakpoints within the file are cleared.
           std::string unconditional = "";
           for (octave_idx_type i = 0; i < line.numel (); i++)
             {
-              lines [0] = line(i).double_value ();
-              bptab.add_breakpoint (name(i).string_value (), lines,
-                                    (use_cond
-                                     ? cond(i).string_value ()
-                                     : unconditional));
+              // FIXME: This isn't very clear, but add_breakpoints_in_function
+              // requires this argument to be of type bp_table::bp_lines type
+              // which is std::set<int>.
+              lines.clear ();
+              lines.insert (line(i).int_value ());
+              bptab.add_breakpoints_in_function (name(i).string_value (),
+                                                 "", lines,
+                                                 (use_cond
+                                                  ? cond(i).string_value ()
+                                                  : unconditional));
             }
           retval = octave_value (line.numel ());
         }
     }
 
+  // If we add a breakpoint, we also need to reset debug_mode.
+  tw.reset_debug_state ();
+
   return retval;
 }
 
-DEFUN (dbclear, args, ,
-       doc: /* -*- texinfo -*-
-@deftypefn  {} {} dbclear @var{func}
-@deftypefnx {} {} dbclear @var{func} @var{line}
-@deftypefnx {} {} dbclear @var{func} @var{line1} @var{line2} @dots{}
+DEFMETHOD (dbclear, interp, args, ,
+           doc: /* -*- texinfo -*-
+@deftypefn  {} {} dbclear @var{fcn}
+@deftypefnx {} {} dbclear @var{fcn} @var{line}
+@deftypefnx {} {} dbclear @var{fcn} @var{line1} @var{line2} @dots{}
 @deftypefnx {} {} dbclear @var{line} @dots{}
 @deftypefnx {} {} dbclear all
-@deftypefnx {} {} dbclear in @var{func}
-@deftypefnx {} {} dbclear in @var{func} at @var{line}
+@deftypefnx {} {} dbclear in @var{fcn}
+@deftypefnx {} {} dbclear in @var{fcn} at @var{line}
 @deftypefnx {} {} dbclear if @var{event}
-@deftypefnx {} {} dbclear ("@var{func}")
-@deftypefnx {} {} dbclear ("@var{func}", @var{line})
-@deftypefnx {} {} dbclear ("@var{func}", @var{line1}, @var{line2}, @dots{})
-@deftypefnx {} {} dbclear ("@var{func}", @var{line1}, @dots{})
+@deftypefnx {} {} dbclear ("@var{fcn}")
+@deftypefnx {} {} dbclear ("@var{fcn}", @var{line})
+@deftypefnx {} {} dbclear ("@var{fcn}", @var{line1}, @var{line2}, @dots{})
+@deftypefnx {} {} dbclear ("@var{fcn}", @var{line1}, @dots{})
 @deftypefnx {} {} dbclear (@var{line}, @dots{})
 @deftypefnx {} {} dbclear ("all")
-Delete a breakpoint at line number @var{line} in the function @var{func}.
+Delete a breakpoint at line number @var{line} in the function @var{fcn}.
 
 Arguments are
 
 @table @var
-@item func
+@item fcn
 Function name as a string variable.  When already in debug mode this
 argument can be omitted and the current function will be used.
 
@@ -283,7 +288,7 @@ as separate arguments or as a vector.
 
 @item event
 An event such as @code{error}, @code{interrupt}, or @code{warning}
-(@pxref{XREFdbstop,,dbstop} for details).
+(@pxref{XREFdbstop,,@code{dbstop}} for details).
 @end table
 
 When called without a line number specification all breakpoints in the named
@@ -297,16 +302,18 @@ files.
 @end deftypefn */)
 {
   std::string symbol_name = "";  // stays empty for "dbclear if error" etc
-  octave::bp_table::intmap lines;
+  std::string class_name = "";
+  octave::bp_table::bp_lines lines;
   std::string dummy;             // "if" condition -- only used for dbstop
 
-  int nargin = args.length ();
+  octave::tree_evaluator& tw = interp.get_evaluator ();
 
-  octave::bp_table& bptab = octave::__get_bp_table__ ("Fdbclear");
+  octave::bp_table& bptab = tw.get_bp_table ();
 
-  bptab.parse_dbfunction_params ("dbclear", args, symbol_name, lines, dummy);
+  bptab.parse_dbfunction_params ("dbclear", args, symbol_name, class_name,
+                                 lines, dummy);
 
-  if (nargin == 1 && symbol_name == "all")
+  if (args.length () == 1 && symbol_name == "all")
     {
       bptab.remove_all_breakpoints ();
       bptab.dbclear_all_signals ();
@@ -314,16 +321,19 @@ files.
   else
     {
       if (symbol_name != "")
-        bptab.remove_breakpoint (symbol_name, lines);
+        bptab.remove_breakpoints_from_function (symbol_name, lines);
     }
 
+  // If we remove a breakpoint, we also need to reset debug_mode.
+  tw.reset_debug_state ();
+
   return ovl ();
 }
 
 DEFMETHOD (dbstatus, interp, args, nargout,
            doc: /* -*- texinfo -*-
 @deftypefn  {} {} dbstatus
-@deftypefnx {} {} dbstatus @var{func}
+@deftypefnx {} {} dbstatus @var{fcn}
 @deftypefnx {} {@var{bp_list} =} dbstatus @dots{}
 Report the location of active breakpoints.
 
@@ -331,16 +341,16 @@ When called with no input or output arguments, print the list of all
 functions with breakpoints and the line numbers where those breakpoints are
 set.
 
-If a function name @var{func} is specified then only report breakpoints
+If a function name @var{fcn} is specified then only report breakpoints
 for the named function and its subfunctions.
 
 The optional return argument @var{bp_list} is a struct array with the
-following fields.
+following fields:
 
 @table @asis
 @item name
-The name of the function with a breakpoint.  A subfunction, say @code{func2}
-within an m-file, say @file{file.m}, is specified as @code{file>func2}.
+The name of the function with a breakpoint.  A subfunction, say @code{fcn2}
+within an m-file, say @file{file.m}, is specified as @code{file>fcn2}.
 
 @item file
 The name of the m-file where the function code is located.
@@ -374,7 +384,9 @@ The @qcode{"warn"} field is set similarly by @code{dbstop if warning}.
   octave::bp_table::fname_bp_map bp_list;
   std::string symbol_name;
 
-  octave::bp_table& bptab = octave::__get_bp_table__ ("Fdbstatus");
+  octave::tree_evaluator& tw = interp.get_evaluator ();
+
+  octave::bp_table& bptab = tw.get_bp_table ();
 
   if (nargin == 1)
     {
@@ -387,17 +399,17 @@ The @qcode{"warn"} field is set similarly by @code{dbstop if warning}.
     }
   else
     {
-/*
-      if (Vdebugging)
-        {
-          octave_user_code *dbg_fcn = get_user_code ();
-          if (dbg_fcn)
-            {
-              symbol_name = dbg_fcn->name ();
-              fcn_list(0) = symbol_name;
-            }
-        }
-*/
+      /*
+            if (tw.in_debug_repl ())
+              {
+                octave_user_code *dbg_fcn = tw.get_user_code ();
+                if (dbg_fcn)
+                  {
+                    symbol_name = dbg_fcn->name ();
+                    fcn_list(0) = symbol_name;
+                  }
+              }
+      */
 
       bp_list = bptab.get_breakpoint_list (fcn_list);
     }
@@ -473,9 +485,9 @@ The @qcode{"warn"} field is set similarly by @code{dbstop if warning}.
       for (const auto& fnm_bp_p : bp_list)
         {
           std::string filename = fnm_bp_p.first;
-          const char *sub_fun = strchr (filename.c_str (), '>');
-          if (sub_fun)
-            filename = filename.substr(0, sub_fun - filename.c_str ());
+          const char *sub_fcn = strchr (filename.c_str (), '>');
+          if (sub_fcn)
+            filename = filename.substr(0, sub_fcn - filename.c_str ());
           octave_value path_name;
           path_name
             = octave::sys::canonicalize_file_name (help_sys.which (filename));
@@ -495,16 +507,16 @@ The @qcode{"warn"} field is set similarly by @code{dbstop if warning}.
       retmap.assign ("line", line);
       retmap.assign ("cond", cond);
 
-      octave_map ew = bptab.stop_on_err_warn_status (false);
+      const octave_map ew = bptab.stop_on_err_warn_status (false);
       if (ew.isempty ())
         {
           retval = octave_value (retmap);
         }
       else
         {
-          octave_map outer (dim_vector (3,1));
+          octave_map outer (dim_vector (3, 1));
           outer.assign ("bkpt", Cell (retmap));
-          for (octave_map::const_iterator f = ew.begin (); f != ew.end (); f++)
+          for (auto f = ew.begin (); f != ew.end (); f++)
             outer.setfield (f->first, ew.contents (f));
 
           retval = octave_value (outer);
@@ -516,18 +528,28 @@ The @qcode{"warn"} field is set similarly by @code{dbstop if warning}.
 
 /*
 %!test
-%! dbclear all;   # Clear out breakpoints before test
-%! dbstop @ftp/dir;
-%! dbstop @audioplayer/set 70;
-%! dbstop quantile>__quantile__;
-%! dbstop ls;
-%! s = dbstatus;
-%! dbclear all;
-%! assert (s(1).name, "@audioplayer/set>setproperty");
-%! assert (s(2).name, "@ftp/dir");
-%! assert (s(3).name, "ls");
-%! assert (s(4).name, "quantile>__quantile__");
-%! assert (s(2).file(end-10:end), [filesep "@ftp" filesep "dir.m"]);
+%! if (isguirunning ())
+%!   orig_show_dbg = __event_manager_gui_preference__ ("editor/show_dbg_file",
+%!                                                     "0");
+%! endif
+%! unwind_protect
+%!   dbclear all;   # Clear out breakpoints before test
+%!   dbstop @ftp/dir;
+%!   dbstop @audioplayer/set 75;
+%!   dbstop quantile>__quantile__;
+%!   dbstop ls;
+%!   s = dbstatus;
+%!   dbclear all;
+%!   assert (s(1).name, "@audioplayer/set>setproperty");
+%!   assert (s(2).name, "@ftp/dir");
+%!   assert (s(3).name, "ls");
+%!   assert (s(4).name, "quantile>__quantile__");
+%!   assert (s(2).file(end-10:end), [filesep "@ftp" filesep "dir.m"]);
+%! unwind_protect_cleanup
+%!   if (isguirunning ())
+%!     __event_manager_gui_preference__ ("editor/show_dbg_file", orig_show_dbg);
+%!   endif
+%! end_unwind_protect
 */
 
 DEFMETHOD (dbwhere, interp, , ,
@@ -538,40 +560,9 @@ is stopped.
 @seealso{dbstack, dblist, dbstatus, dbcont, dbstep, dbup, dbdown}
 @end deftypefn */)
 {
-  octave_user_code *dbg_fcn = octave::get_user_code ();
+  octave::tree_evaluator& tw = interp.get_evaluator ();
 
-  if (! dbg_fcn)
-    {
-      octave_stdout << "stopped at top level" << std::endl;
-      return ovl ();
-    }
-
-  octave_stdout << "stopped in " << dbg_fcn->name () << " at ";
-
-  octave::call_stack& cs = interp.get_call_stack ();
-
-  int l = cs.debug_user_code_line ();
-
-  if (l > 0)
-    {
-      octave_stdout << "line " << l;
-
-      std::string file_name = dbg_fcn->fcn_file_name ();
-
-      if (! file_name.empty ())
-        {
-          octave_stdout << " [" << file_name << ']' << std::endl;
-
-          std::string line = dbg_fcn->get_code_line (l);
-
-          if (! line.empty ())
-            octave_stdout << l << ": " << line << std::endl;
-        }
-      else
-        octave_stdout << std::endl;
-    }
-  else
-    octave_stdout << "<unknown line>" << std::endl;
+  tw.debug_where (octave_stdout);
 
   return ovl ();
 }
@@ -579,13 +570,13 @@ is stopped.
 static void
 do_dbtype (std::ostream& os, const std::string& name, int start, int end)
 {
-  std::string ff = fcn_file_in_path (name);
+  std::string ff = octave::fcn_file_in_path (name);
 
   if (ff.empty ())
     os << "dbtype: unknown function " << name << "\n";
   else
     {
-      std::ifstream fs (ff.c_str (), std::ios::in);
+      std::ifstream fs = octave::sys::ifstream (ff.c_str (), std::ios::in);
 
       if (! fs)
         os << "dbtype: unable to open '" << ff << "' for reading!\n";
@@ -607,16 +598,16 @@ do_dbtype (std::ostream& os, const std::string& name, int start, int end)
   os.flush ();
 }
 
-DEFUN (dbtype, args, ,
-       doc: /* -*- texinfo -*-
+DEFMETHOD (dbtype, interp, args, ,
+           doc: /* -*- texinfo -*-
 @deftypefn  {} {} dbtype
 @deftypefnx {} {} dbtype @var{lineno}
 @deftypefnx {} {} dbtype @var{startl:endl}
 @deftypefnx {} {} dbtype @var{startl:end}
-@deftypefnx {} {} dbtype @var{func}
-@deftypefnx {} {} dbtype @var{func} @var{lineno}
-@deftypefnx {} {} dbtype @var{func} @var{startl:endl}
-@deftypefnx {} {} dbtype @var{func} @var{startl:end}
+@deftypefnx {} {} dbtype @var{fcn}
+@deftypefnx {} {} dbtype @var{fcn} @var{lineno}
+@deftypefnx {} {} dbtype @var{fcn} @var{startl:endl}
+@deftypefnx {} {} dbtype @var{fcn} @var{startl:end}
 Display a script file with line numbers.
 
 When called with no arguments in debugging mode, display the script file
@@ -635,10 +626,12 @@ numbers.
 
   string_vector argv = args.make_argv ("dbtype");
 
+  octave::tree_evaluator& tw = interp.get_evaluator ();
+
   switch (args.length ())
     {
     case 0:  // dbtype
-      dbg_fcn = octave::get_user_code ();
+      dbg_fcn = tw.get_user_code ();
 
       if (! dbg_fcn)
         error ("dbtype: must be inside a user function to give no arguments to dbtype\n");
@@ -648,15 +641,15 @@ numbers.
 
       break;
 
-    case 1:  // (dbtype start:end) || (dbtype func) || (dbtype lineno)
+    case 1:  // (dbtype start:end) || (dbtype fcn) || (dbtype lineno)
       {
         std::string arg = argv[1];
 
-        size_t ind = arg.find (':');
+        std::size_t ind = arg.find (':');
 
         if (ind != std::string::npos)  // (dbtype start:end)
           {
-            dbg_fcn = octave::get_user_code ();
+            dbg_fcn = tw.get_user_code ();
 
             if (dbg_fcn)
               {
@@ -680,13 +673,13 @@ numbers.
                            start, end);
               }
           }
-        else  // (dbtype func) || (dbtype lineno)
+        else  // (dbtype fcn) || (dbtype lineno)
           {
             int line = atoi (arg.c_str ());
 
-            if (line == 0)  // (dbtype func)
+            if (line == 0)  // (dbtype fcn)
               {
-                dbg_fcn = octave::get_user_code (arg);
+                dbg_fcn = tw.get_user_code (arg);
 
                 if (! dbg_fcn)
                   error ("dbtype: function <%s> not found\n", arg.c_str ());
@@ -699,7 +692,7 @@ numbers.
                 if (line <= 0)
                   error ("dbtype: start and end lines must be >= 1\n");
 
-                dbg_fcn = octave::get_user_code ();
+                dbg_fcn = tw.get_user_code ();
 
                 if (dbg_fcn)
                   do_dbtype (octave_stdout, dbg_fcn->fcn_file_name (),
@@ -709,16 +702,16 @@ numbers.
       }
       break;
 
-    case 2:  // (dbtype func start:end) || (dbtype func start)
+    case 2:  // (dbtype fcn start:end) || (dbtype fcn start)
       {
-        dbg_fcn = octave::get_user_code (argv[1]);
+        dbg_fcn = tw.get_user_code (argv[1]);
 
         if (! dbg_fcn)
           error ("dbtype: function <%s> not found\n", argv[1].c_str ());
 
         std::string arg = argv[2];
         int start, end;
-        size_t ind = arg.find (':');
+        std::size_t ind = arg.find (':');
 
         if (ind != std::string::npos)
           {
@@ -784,7 +777,9 @@ If unspecified @var{n} defaults to 10 (+/- 5 lines)
         error ("dblist: N must be a non-negative integer");
     }
 
-  octave_user_code *dbg_fcn = octave::get_user_code ();
+  octave::tree_evaluator& tw = interp.get_evaluator ();
+
+  octave_user_code *dbg_fcn = tw.get_user_code ();
 
   if (! dbg_fcn)
     error ("dblist: must be inside a user function to use dblist\n");
@@ -799,9 +794,7 @@ If unspecified @var{n} defaults to 10 (+/- 5 lines)
       name = dbg_fcn->name ();
     }
 
-  octave::call_stack& cs = interp.get_call_stack ();
-
-  int l = cs.debug_user_code_line ();
+  int l = tw.debug_user_code_line ();
 
   if (l > 0)
     {
@@ -839,11 +832,9 @@ do_dbstack (octave::interpreter& interp, const octave_value_list& args,
 
   octave_value_list retval;
 
-  octave::unwind_protect frame;
-
   octave_idx_type curr_frame = -1;
 
-  size_t nskip = 0;
+  octave_idx_type nskip = 0;
 
   if (nargin == 1 || nargin == 2)
     {
@@ -866,7 +857,7 @@ do_dbstack (octave::interpreter& interp, const octave_value_list& args,
           else
             n = arg.int_value ();
 
-          if (n <= 0)
+          if (n < 0)
             error ("dbstack: N must be a non-negative integer");
         }
 
@@ -874,14 +865,14 @@ do_dbstack (octave::interpreter& interp, const octave_value_list& args,
         nskip = n;
     }
 
-  octave::call_stack& cs = interp.get_call_stack ();
+  octave::tree_evaluator& tw = interp.get_evaluator ();
 
   if (nargout == 0)
     {
-      octave_map stk = cs.backtrace (nskip, curr_frame);
-      octave_idx_type nframes_to_display = stk.numel ();
+      octave_map stk = tw.backtrace (curr_frame);
+      octave_idx_type nframes = stk.numel ();
 
-      if (nframes_to_display > 0)
+      if (nframes > 0)
         {
           octave::preserve_stream_state stream_state (os);
 
@@ -893,16 +884,16 @@ do_dbstack (octave::interpreter& interp, const octave_value_list& args,
 
           bool show_top_level = true;
 
-          size_t max_name_len = 0;
+          std::size_t max_name_len = 0;
 
-          for (octave_idx_type i = 0; i < nframes_to_display; i++)
+          for (octave_idx_type i = nskip; i < nframes; i++)
             {
               std::string name = names(i).string_value ();
 
               max_name_len = std::max (name.length (), max_name_len);
             }
 
-          for (octave_idx_type i = 0; i < nframes_to_display; i++)
+          for (octave_idx_type i = nskip; i < nframes; i++)
             {
               std::string name = names(i).string_value ();
               std::string file = files(i).string_value ();
@@ -918,15 +909,25 @@ do_dbstack (octave::interpreter& interp, const octave_value_list& args,
                  << std::endl;
             }
 
-          if (show_top_level)
+          if (tw.at_top_level () && show_top_level)
             os << "  --> top level" << std::endl;
         }
     }
   else
     {
-      octave_map stk = cs.backtrace (nskip, curr_frame, false);
+      octave_map stk = tw.backtrace (curr_frame, false);
 
-      retval = ovl (stk, curr_frame < 0 ? 1 : curr_frame + 1);
+      octave_idx_type num_skip = std::min (nskip, stk.numel ());
+
+      idx_vector first = idx_vector (0);
+
+      for (octave_idx_type i = 0; i < num_skip; i++)
+        stk.delete_elements (first);
+
+      curr_frame -= num_skip;
+      curr_frame = (curr_frame < 0 ? 0 : curr_frame + 1);
+
+      retval = ovl (stk, curr_frame);
     }
 
   return retval;
@@ -940,7 +941,7 @@ do_dbstack (octave::interpreter& interp, const octave_value_list& args,
 void
 show_octave_dbstack (void)
 {
-  do_dbstack (octave::__get_interpreter__ ("show_octave_dbstack"),
+  do_dbstack (octave::__get_interpreter__ (),
               octave_value_list (), 0, std::cerr);
 }
 
@@ -957,8 +958,8 @@ With optional argument @var{n}, omit the @var{n} innermost stack frames.
 Although accepted, the argument @var{-completenames} is silently ignored.
 Octave always returns absolute filenames.
 
-The arguments @var{n} and @var{-completenames} can be both specified in any
-order.
+The arguments @var{n} and @var{-completenames} can both be specified and may
+appear in any order.
 
 The optional return argument @var{stack} is a struct array with the
 following fields:
@@ -976,11 +977,6 @@ The line number of an active breakpoint.
 @item column
 The column number of the line where the breakpoint begins.
 
-@item scope
-Undocumented.
-
-@item context
-Undocumented.
 @end table
 
 The return argument @var{idx} specifies which element of the @var{stack}
@@ -1014,10 +1010,9 @@ do_dbupdown (octave::interpreter& interp, const octave_value_list& args,
   if (who == "dbup")
     n = -n;
 
-  octave::call_stack& cs = interp.get_call_stack ();
+  octave::tree_evaluator& tw = interp.get_evaluator ();
 
-  if (! cs.goto_frame_relative (n, true))
-    error ("%s: invalid stack frame", who.c_str ());
+  tw.dbupdown (n, true);
 }
 
 DEFMETHOD (dbup, interp, args, ,
@@ -1050,8 +1045,8 @@ If @var{n} is omitted, move down one frame.
   return ovl ();
 }
 
-DEFUN (dbstep, args, ,
-       doc: /* -*- texinfo -*-
+DEFMETHOD (dbstep, interp, args, ,
+           doc: /* -*- texinfo -*-
 @deftypefn  {} {} dbstep
 @deftypefnx {} {} dbstep @var{n}
 @deftypefnx {} {} dbstep in
@@ -1073,108 +1068,111 @@ function returns.
 @seealso{dbcont, dbquit}
 @end deftypefn */)
 {
-  if (! Vdebugging)
+  octave::tree_evaluator& tw = interp.get_evaluator ();
+
+  if (! tw.in_debug_repl ())
     error ("dbstep: can only be called in debug mode");
 
   int nargin = args.length ();
 
+  if (nargin > 1)
+    print_usage ();
+
+  int n = 0;
+
+  if (nargin == 1)
+    {
+      std::string arg
+        = args(0).xstring_value ("dbstep: input argument must be a string");
+
+      if (arg == "in")
+        n = -1;
+      else if (arg == "out")
+        n = -2;
+      else
+        {
+          n = atoi (arg.c_str ());
+
+          if (n < 1)
+            error ("dbstep: invalid argument");
+        }
+    }
+  else
+    n = 1;
+
+  if (n != 0)
+    {
+      tw.set_dbstep_flag (n);
+
+      // If we set the dbstep flag, we also need to reset debug_mode.
+      tw.reset_debug_state ();
+
+    }
+
+  return ovl ();
+}
+
+DEFALIAS (dbnext, dbstep);
+
+DEFMETHOD (dbcont, interp, args, ,
+           doc: /* -*- texinfo -*-
+@deftypefn {} {} dbcont
+Leave command-line debugging mode and continue code execution normally.
+@seealso{dbstep, dbquit}
+@end deftypefn */)
+{
+  octave::tree_evaluator& tw = interp.get_evaluator ();
+
+  if (! tw.in_debug_repl ())
+    error ("dbcont: can only be called in debug mode");
+
+  if (args.length () != 0)
+    print_usage ();
+
+  tw.dbcont ();
+
+  return ovl ();
+}
+
+DEFMETHOD (dbquit, interp, args, ,
+           doc: /* -*- texinfo -*-
+@deftypefn  {} {} dbquit
+@deftypefnx {} {} dbquit all
+Quit debugging mode immediately without further code execution.  With no
+arguments, exit the current debugging level.  With argument @code{all},
+exit all debugging levels and return to the Octave prompt.
+@seealso{dbcont, dbstep}
+@end deftypefn */)
+{
+  octave::tree_evaluator& tw = interp.get_evaluator ();
+
+  if (! tw.in_debug_repl ())
+    error ("dbquit: can only be called in debug mode");
+
+  int nargin = args.length ();
+
   if (nargin > 1)
     print_usage ();
 
   if (nargin == 1)
     {
-      std::string arg = args(0).xstring_value ("dbstep: input argument must be a string");
+      std::string arg
+        = args(0).xstring_value ("dbquit: input argument must be a string");
 
-      if (arg == "in")
-        {
-          Vdebugging = false;
-          Vtrack_line_num = true;
-
-          octave::tree_evaluator::dbstep_flag = -1;
-        }
-      else if (arg == "out")
-        {
-          Vdebugging = false;
-          Vtrack_line_num = true;
-
-          octave::tree_evaluator::dbstep_flag = -2;
-        }
+      if (arg == "all")
+        tw.dbquit (true);
       else
-        {
-          int n = atoi (arg.c_str ());
-
-          if (n < 1)
-            error ("dbstep: invalid argument");
-
-          Vdebugging = false;
-          Vtrack_line_num = true;
-
-          octave::tree_evaluator::dbstep_flag = n;
-        }
+        error ("dbquit: unrecognized argument '%s'", arg.c_str ());
     }
   else
-    {
-      Vdebugging = false;
-      Vtrack_line_num = true;
-
-      octave::tree_evaluator::dbstep_flag = 1;
-    }
+    tw.dbquit ();
 
   return ovl ();
 }
 
-DEFALIAS (dbnext, dbstep);
-
-DEFUN (dbcont, args, ,
-       doc: /* -*- texinfo -*-
-@deftypefn {} {} dbcont
-Leave command-line debugging mode and continue code execution normally.
-@seealso{dbstep, dbquit}
-@end deftypefn */)
-{
-  if (! Vdebugging)
-    error ("dbcont: can only be called in debug mode");
-
-  if (args.length () != 0)
-    print_usage ();
-
-  Vdebugging = false;
-  Vtrack_line_num = true;
-
-  octave::tree_evaluator::reset_debug_state ();
-
-  return ovl ();
-}
-
-DEFUN (dbquit, args, ,
-       doc: /* -*- texinfo -*-
-@deftypefn {} {} dbquit
-Quit debugging mode immediately without further code execution and return to
-the Octave prompt.
-@seealso{dbcont, dbstep}
-@end deftypefn */)
-{
-  if (! Vdebugging)
-    error ("dbquit: can only be called in debug mode");
-
-  if (args.length () != 0)
-    print_usage ();
-
-  // FIXME: there are too many debug mode flags!
-
-  Vdebugging = false;
-
-  octave::tree_evaluator::reset_debug_state ();
-  octave::tree_evaluator::debug_mode = false;
-
-  throw octave::interrupt_exception ();
-
-  return ovl ();
-}
-
-DEFUN (isdebugmode, args, ,
-       doc: /* -*- texinfo -*-
-@deftypefn {} {} isdebugmode ()
+DEFMETHOD (isdebugmode, interp, args, ,
+           doc: /* -*- texinfo -*-
+@deftypefn {} {@var{tf} =} isdebugmode ()
 Return true if in debugging mode, otherwise false.
 @seealso{dbwhere, dbstack, dbstatus}
 @end deftypefn */)
@@ -1182,11 +1180,13 @@ Return true if in debugging mode, otherwise false.
   if (args.length () != 0)
     print_usage ();
 
-  return ovl (Vdebugging);
+  octave::tree_evaluator& tw = interp.get_evaluator ();
+
+  return ovl (tw.in_debug_repl ());
 }
 
-DEFUN (__db_next_breakpoint_quiet__, args, ,
-       doc: /* -*- texinfo -*-
+DEFMETHOD (__db_next_breakpoint_quiet__, interp, args, ,
+           doc: /* -*- texinfo -*-
 @deftypefn  {} {} __db_next_breakpoint_quiet__ ()
 @deftypefnx {} {} __db_next_breakpoint_quiet__ (@var{flag})
 Disable line info printing at the next breakpoint.
@@ -1204,7 +1204,11 @@ With a logical argument @var{flag}, set the state on or off.
   if (nargin == 1)
     state = args(0).bool_value ();
 
-  octave::tree_evaluator::quiet_breakpoint_flag = state;
+  octave::tree_evaluator& tw = interp.get_evaluator ();
+
+  tw.quiet_breakpoint_flag (state);
 
   return ovl ();
 }
+
+OCTAVE_END_NAMESPACE(octave)
