@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1996-2022 The Octave Project Developers
+// Copyright (C) 1996-2023 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -98,8 +98,7 @@ std::string btyp_class_name[btyp_num_types+1] =
 DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (octave_base_value,
                                      "<unknown type>", "unknown");
 
-// TRUE means to perform automatic sparse to real mutation if there
-// is memory to be saved
+// DEPRECATED in Octave 8.
 bool Vsparse_auto_mutate = false;
 
 octave_base_value *
@@ -396,7 +395,7 @@ void
 octave_base_value::convert_to_row_or_column_vector (void)
 {
   err_wrong_type_arg
-    ("octave_base_value::convert_to_row_or_column_vector ()", type_name ());
+  ("octave_base_value::convert_to_row_or_column_vector ()", type_name ());
 }
 
 void
@@ -475,11 +474,13 @@ octave_base_value::print_info (std::ostream& os,
         err_wrong_type_arg (ee, "octave_base_value::" #F "_value ()", type_name ()); \
       }                                                                 \
                                                                         \
+    static const double out_of_range_top                                \
+      = static_cast<double>(std::numeric_limits<T>::max ()) + 1.;       \
     if (require_int && octave::math::x_nint (d) != d)                   \
       error_with_cfn ("conversion of %g to " #T " value failed", d);    \
     else if (d < std::numeric_limits<T>::min ())                        \
       retval = std::numeric_limits<T>::min ();                          \
-    else if (d > std::numeric_limits<T>::max ())                        \
+    else if (d >= out_of_range_top)                                     \
       retval = std::numeric_limits<T>::max ();                          \
     else                                                                \
       retval = static_cast<T> (octave::math::fix (d));                  \
@@ -811,18 +812,22 @@ octave_base_value::cellstr_value (void) const
   err_wrong_type_arg ("octave_base_value::cellstr_value()", type_name ());
 }
 
-octave::range<float>
-octave_base_value::float_range_value (void) const
-{
-  err_wrong_type_arg ("octave_base_value::float_range_value()", type_name ());
-}
-
 octave::range<double>
 octave_base_value::range_value (void) const
 {
   err_wrong_type_arg ("octave_base_value::range_value()", type_name ());
 }
 
+// For now, disable all but range<double>.
+
+#if 0
+
+octave::range<float>
+octave_base_value::float_range_value (void) const
+{
+  err_wrong_type_arg ("octave_base_value::float_range_value()", type_name ());
+}
+
 octave::range<octave_int8>
 octave_base_value::int8_range_value (void) const
 {
@@ -871,6 +876,8 @@ octave_base_value::uint64_range_value (void) const
   err_wrong_type_arg ("octave_base_value::uint64_range_value()", type_name ());
 }
 
+#endif
+
 octave_map
 octave_base_value::map_value (void) const
 {
@@ -894,6 +901,12 @@ octave_base_value::map_keys (void) const
   err_wrong_type_arg ("octave_base_value::map_keys()", type_name ());
 }
 
+bool
+octave_base_value::isfield (const std::string&) const
+{
+  err_wrong_type_arg ("octave_base_value::isfield()", type_name ());
+}
+
 std::size_t
 octave_base_value::nparents (void) const
 {
@@ -1043,7 +1056,7 @@ octave_base_value::sort (octave_idx_type, sortmode) const
 }
 
 octave_value
-octave_base_value::sort (Array<octave_idx_type> &,
+octave_base_value::sort (Array<octave_idx_type>&,
                          octave_idx_type, sortmode) const
 {
   err_wrong_type_arg ("octave_base_value::sort ()", type_name ());
@@ -1144,18 +1157,18 @@ void
 octave_base_value::warn_load (const char *type) const
 {
   warning_with_id
-    ("Octave:load-save-unavailable",
-     "%s: loading %s files not available in this version of Octave",
-     t_name.c_str (), type);
+  ("Octave:load-save-unavailable",
+   "%s: loading %s files not available in this version of Octave",
+   t_name.c_str (), type);
 }
 
 void
 octave_base_value::warn_save (const char *type) const
 {
   warning_with_id
-    ("Octave:load-save-unavailable",
-     "%s: saving %s files not available in this version of Octave",
-     t_name.c_str (), type);
+  ("Octave:load-save-unavailable",
+   "%s: saving %s files not available in this version of Octave",
+   t_name.c_str (), type);
 }
 
 void
@@ -1186,9 +1199,10 @@ octave_value
 octave_base_value::dump (void) const
 {
   std::map<std::string, octave_value> m
-    = {{ "class", this->class_name () },
-       { "type", this->type_name () },
-       { "dims", this->dims().as_array () }};
+  = {{ "class", this->class_name () },
+    { "type", this->type_name () },
+    { "dims", this->dims().as_array () }
+  };
 
   return octave_value (m);
 }
@@ -1231,8 +1245,7 @@ octave_base_value::numeric_assign (const std::string& type,
   int t_lhs = type_id ();
   int t_rhs = rhs.type_id ();
 
-  octave::type_info& ti
-    = octave::__get_type_info__ ("octave_base_value::numeric_assign");
+  octave::type_info& ti = octave::__get_type_info__ ();
 
   octave::type_info::assign_op_fcn f
     = ti.lookup_assign_op (octave_value::op_asn_eq, t_lhs, t_rhs);
@@ -1296,7 +1309,7 @@ octave_base_value::numeric_assign (const std::string& type,
                    && (ti.lookup_assign_op (octave_value::op_asn_eq,
                                             cf_this.type_id (), t_rhs)
                        || ti.lookup_pref_assign_conv (cf_this.type_id (),
-                                                      t_rhs) >= 0))
+                           t_rhs) >= 0))
             cf_rhs = nullptr;
 
           if (cf_rhs)
@@ -1355,7 +1368,7 @@ bool octave_base_value::s_beginning_of_line = true;
 void
 octave_base_value::indent (std::ostream& os) const
 {
-  assert (s_curr_print_indent_level >= 0);
+  panic_unless (s_curr_print_indent_level >= 0);
 
   if (s_beginning_of_line)
     {
@@ -1510,8 +1523,7 @@ make_idx_args (const std::string& type,
 bool
 called_from_builtin (void)
 {
-  octave::tree_evaluator& tw
-    = octave::__get_evaluator__ ("called_from_builtin");
+  octave::tree_evaluator& tw = octave::__get_evaluator__ ();
 
   octave_function *fcn = tw.caller_function ();
 
@@ -1524,7 +1536,7 @@ called_from_builtin (void)
   return (fcn && fcn->name () == "builtin");
 }
 
-OCTAVE_NAMESPACE_BEGIN
+OCTAVE_BEGIN_NAMESPACE(octave)
 
 void
 install_base_type_conversions (octave::type_info& ti)
@@ -1547,49 +1559,4 @@ install_base_type_conversions (octave::type_info& ti)
   INSTALL_WIDENOP_TI (ti, octave_base_value, octave_cell, cell_conv);
 }
 
-DEFUN (sparse_auto_mutate, args, nargout,
-       doc: /* -*- texinfo -*-
-@deftypefn  {} {@var{val} =} sparse_auto_mutate ()
-@deftypefnx {} {@var{old_val} =} sparse_auto_mutate (@var{new_val})
-@deftypefnx {} {} sparse_auto_mutate (@var{new_val}, "local")
-Query or set the internal variable that controls whether Octave will
-automatically mutate sparse matrices to full matrices to save memory.
-
-For example:
-
-@example
-@group
-s = speye (3);
-sparse_auto_mutate (false);
-s(:, 1) = 1;
-typeinfo (s)
-@result{} sparse matrix
-sparse_auto_mutate (true);
-s(1, :) = 1;
-typeinfo (s)
-@result{} matrix
-@end group
-@end example
-
-When called from inside a function with the @qcode{"local"} option, the
-variable is changed locally for the function and any subroutines it calls.
-The original variable value is restored when exiting the function.
-@end deftypefn */)
-{
-  return set_internal_variable (Vsparse_auto_mutate, args, nargout,
-                                "sparse_auto_mutate");
-}
-
-/*
-%!test
-%! s = speye (3);
-%! sparse_auto_mutate (false);
-%! s(:, 1) = 1;
-%! assert (typeinfo (s), "sparse matrix");
-%! sparse_auto_mutate (true);
-%! s(1, :) = 1;
-%! assert (typeinfo (s), "matrix");
-%! sparse_auto_mutate (false);
-*/
-
-OCTAVE_NAMESPACE_END
+OCTAVE_END_NAMESPACE(octave)
