@@ -1,32 +1,36 @@
-/*
-
-Copyright (C) 1994-2016 John W. Eaton
-Copyright (C) 2016 Barbara Lócsi
-
-This file is part of Octave.
-
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1994-2023 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
 #endif
 
+#include "Array.h"
 #include "EIG.h"
 #include "dColVector.h"
+#include "dMatrix.h"
 #include "lo-error.h"
 #include "lo-lapack-proto.h"
 
@@ -37,15 +41,16 @@ EIG::init (const Matrix& a, bool calc_rev, bool calc_lev, bool balance)
     (*current_liboctave_error_handler)
       ("EIG: matrix contains Inf or NaN values");
 
-  if (a.is_symmetric ())
+  if (a.issymmetric ())
     return symmetric_init (a, calc_rev, calc_lev);
 
-  octave_idx_type n = a.rows ();
+  F77_INT n = octave::to_f77_int (a.rows ());
+  F77_INT a_nc = octave::to_f77_int (a.cols ());
 
-  if (n != a.cols ())
+  if (n != a_nc)
     (*current_liboctave_error_handler) ("EIG requires square matrix");
 
-  octave_idx_type info = 0;
+  F77_INT info = 0;
 
   Matrix atmp = a;
   double *tmp_data = atmp.fortran_vec ();
@@ -56,19 +61,19 @@ EIG::init (const Matrix& a, bool calc_rev, bool calc_lev, bool balance)
   Array<double> wi (dim_vector (n, 1));
   double *pwi = wi.fortran_vec ();
 
-  octave_idx_type tnvr = calc_rev ? n : 0;
+  F77_INT tnvr = (calc_rev ? n : 0);
   Matrix vr (tnvr, tnvr);
   double *pvr = vr.fortran_vec ();
 
-  octave_idx_type tnvl = calc_lev ? n : 0;
+  F77_INT tnvl = (calc_lev ? n : 0);
   Matrix vl (tnvl, tnvl);
   double *pvl = vl.fortran_vec ();
 
-  octave_idx_type lwork = -1;
+  F77_INT lwork = -1;
   double dummy_work;
 
-  octave_idx_type ilo;
-  octave_idx_type ihi;
+  F77_INT ilo;
+  F77_INT ihi;
 
   Array<double> scale (dim_vector (n, 1));
   double *pscale = scale.fortran_vec ();
@@ -81,7 +86,7 @@ EIG::init (const Matrix& a, bool calc_rev, bool calc_lev, bool balance)
   Array<double> rcondv (dim_vector (n, 1));
   double *prcondv = rcondv.fortran_vec ();
 
-  octave_idx_type dummy_iwork;
+  F77_INT dummy_iwork;
 
   F77_XFCN (dgeevx, DGEEVX, (F77_CONST_CHAR_ARG2 (balance ? "B" : "N", 1),
                              F77_CONST_CHAR_ARG2 (calc_lev ? "V" : "N", 1),
@@ -99,7 +104,7 @@ EIG::init (const Matrix& a, bool calc_rev, bool calc_lev, bool balance)
   if (info != 0)
     (*current_liboctave_error_handler) ("dgeevx workspace query failed");
 
-  lwork = static_cast<octave_idx_type> (dummy_work);
+  lwork = static_cast<F77_INT> (dummy_work);
   Array<double> work (dim_vector (lwork, 1));
   double *pwork = work.fortran_vec ();
 
@@ -122,45 +127,45 @@ EIG::init (const Matrix& a, bool calc_rev, bool calc_lev, bool balance)
   if (info > 0)
     (*current_liboctave_error_handler) ("dgeevx failed to converge");
 
-  lambda.resize (n);
-  octave_idx_type nvr = calc_rev ? n : 0;
-  v.resize (nvr, nvr);
-  octave_idx_type nvl = calc_lev ? n : 0;
-  w.resize (nvl, nvl);
+  m_lambda.resize (n);
+  F77_INT nvr = (calc_rev ? n : 0);
+  m_v.resize (nvr, nvr);
+  F77_INT nvl = (calc_lev ? n : 0);
+  m_w.resize (nvl, nvl);
 
-  for (octave_idx_type j = 0; j < n; j++)
+  for (F77_INT j = 0; j < n; j++)
     {
       if (wi.elem (j) == 0.0)
         {
-          lambda.elem (j) = Complex (wr.elem (j));
-          for (octave_idx_type i = 0; i < nvr; i++)
-            v.elem (i, j) = vr.elem (i, j);
+          m_lambda.elem (j) = Complex (wr.elem (j));
+          for (F77_INT i = 0; i < nvr; i++)
+            m_v.elem (i, j) = vr.elem (i, j);
 
-          for (octave_idx_type i = 0; i < nvl; i++)
-            w.elem (i, j) = vl.elem (i, j);
+          for (F77_INT i = 0; i < nvl; i++)
+            m_w.elem (i, j) = vl.elem (i, j);
         }
       else
         {
           if (j+1 >= n)
             (*current_liboctave_error_handler) ("EIG: internal error");
 
-          lambda.elem (j) = Complex (wr.elem (j), wi.elem (j));
-          lambda.elem (j+1) = Complex (wr.elem (j+1), wi.elem (j+1));
+          m_lambda.elem (j) = Complex (wr.elem (j), wi.elem (j));
+          m_lambda.elem (j+1) = Complex (wr.elem (j+1), wi.elem (j+1));
 
-          for (octave_idx_type i = 0; i < nvr; i++)
+          for (F77_INT i = 0; i < nvr; i++)
             {
               double real_part = vr.elem (i, j);
               double imag_part = vr.elem (i, j+1);
-              v.elem (i, j) = Complex (real_part, imag_part);
-              v.elem (i, j+1) = Complex (real_part, -imag_part);
+              m_v.elem (i, j) = Complex (real_part, imag_part);
+              m_v.elem (i, j+1) = Complex (real_part, -imag_part);
             }
 
-          for (octave_idx_type i = 0; i < nvl; i++)
+          for (F77_INT i = 0; i < nvl; i++)
             {
               double real_part = vl.elem (i, j);
               double imag_part = vl.elem (i, j+1);
-              w.elem (i, j) = Complex (real_part, imag_part);
-              w.elem (i, j+1) = Complex (real_part, -imag_part);
+              m_w.elem (i, j) = Complex (real_part, imag_part);
+              m_w.elem (i, j+1) = Complex (real_part, -imag_part);
             }
           j++;
         }
@@ -172,12 +177,13 @@ EIG::init (const Matrix& a, bool calc_rev, bool calc_lev, bool balance)
 octave_idx_type
 EIG::symmetric_init (const Matrix& a, bool calc_rev, bool calc_lev)
 {
-  octave_idx_type n = a.rows ();
+  F77_INT n = octave::to_f77_int (a.rows ());
+  F77_INT a_nc = octave::to_f77_int (a.cols ());
 
-  if (n != a.cols ())
+  if (n != a_nc)
     (*current_liboctave_error_handler) ("EIG requires square matrix");
 
-  octave_idx_type info = 0;
+  F77_INT info = 0;
 
   Matrix atmp = a;
   double *tmp_data = atmp.fortran_vec ();
@@ -185,7 +191,7 @@ EIG::symmetric_init (const Matrix& a, bool calc_rev, bool calc_lev)
   ColumnVector wr (n);
   double *pwr = wr.fortran_vec ();
 
-  octave_idx_type lwork = -1;
+  F77_INT lwork = -1;
   double dummy_work;
 
   F77_XFCN (dsyev, DSYEV, (F77_CONST_CHAR_ARG2 (calc_rev ? "V" : "N", 1),
@@ -197,7 +203,7 @@ EIG::symmetric_init (const Matrix& a, bool calc_rev, bool calc_lev)
   if (info != 0)
     (*current_liboctave_error_handler) ("dsyev workspace query failed");
 
-  lwork = static_cast<octave_idx_type> (dummy_work);
+  lwork = static_cast<F77_INT> (dummy_work);
   Array<double> work (dim_vector (lwork, 1));
   double *pwork = work.fortran_vec ();
 
@@ -213,9 +219,9 @@ EIG::symmetric_init (const Matrix& a, bool calc_rev, bool calc_lev)
   if (info > 0)
     (*current_liboctave_error_handler) ("dsyev failed to converge");
 
-  lambda = ComplexColumnVector (wr);
-  v = calc_rev ? ComplexMatrix (atmp) : ComplexMatrix ();
-  w = calc_lev ? ComplexMatrix (atmp) : ComplexMatrix ();
+  m_lambda = ComplexColumnVector (wr);
+  m_v = (calc_rev ? ComplexMatrix (atmp) : ComplexMatrix ());
+  m_w = (calc_lev ? ComplexMatrix (atmp) : ComplexMatrix ());
 
   return info;
 }
@@ -227,15 +233,16 @@ EIG::init (const ComplexMatrix& a, bool calc_rev, bool calc_lev, bool balance)
     (*current_liboctave_error_handler)
       ("EIG: matrix contains Inf or NaN values");
 
-  if (a.is_hermitian ())
+  if (a.ishermitian ())
     return hermitian_init (a, calc_rev, calc_lev);
 
-  octave_idx_type n = a.rows ();
+  F77_INT n = octave::to_f77_int (a.rows ());
+  F77_INT a_nc = octave::to_f77_int (a.cols ());
 
-  if (n != a.cols ())
+  if (n != a_nc)
     (*current_liboctave_error_handler) ("EIG requires square matrix");
 
-  octave_idx_type info = 0;
+  F77_INT info = 0;
 
   ComplexMatrix atmp = a;
   Complex *tmp_data = atmp.fortran_vec ();
@@ -243,23 +250,23 @@ EIG::init (const ComplexMatrix& a, bool calc_rev, bool calc_lev, bool balance)
   ComplexColumnVector wr (n);
   Complex *pw = wr.fortran_vec ();
 
-  octave_idx_type nvr = calc_rev ? n : 0;
+  F77_INT nvr = (calc_rev ? n : 0);
   ComplexMatrix vrtmp (nvr, nvr);
   Complex *pvr = vrtmp.fortran_vec ();
 
-  octave_idx_type nvl = calc_lev ? n : 0;
+  F77_INT nvl = (calc_lev ? n : 0);
   ComplexMatrix vltmp (nvl, nvl);
   Complex *pvl = vltmp.fortran_vec ();
 
-  octave_idx_type lwork = -1;
+  F77_INT lwork = -1;
   Complex dummy_work;
 
-  octave_idx_type lrwork = 2*n;
+  F77_INT lrwork = 2*n;
   Array<double> rwork (dim_vector (lrwork, 1));
   double *prwork = rwork.fortran_vec ();
 
-  octave_idx_type ilo;
-  octave_idx_type ihi;
+  F77_INT ilo;
+  F77_INT ihi;
 
   Array<double> scale (dim_vector (n, 1));
   double *pscale = scale.fortran_vec ();
@@ -277,10 +284,11 @@ EIG::init (const ComplexMatrix& a, bool calc_rev, bool calc_lev, bool balance)
                              F77_CONST_CHAR_ARG2 (calc_rev ? "V" : "N", 1),
                              F77_CONST_CHAR_ARG2 ("N", 1),
                              n, F77_DBLE_CMPLX_ARG (tmp_data), n,
-                             F77_DBLE_CMPLX_ARG (pw), F77_DBLE_CMPLX_ARG (pvl), n,
-                             F77_DBLE_CMPLX_ARG (pvr), n, ilo, ihi, pscale, abnrm,
-                             prconde, prcondv,
-                             F77_DBLE_CMPLX_ARG (&dummy_work), lwork, prwork, info
+                             F77_DBLE_CMPLX_ARG (pw), F77_DBLE_CMPLX_ARG (pvl),
+                             n, F77_DBLE_CMPLX_ARG (pvr), n, ilo, ihi,
+                             pscale, abnrm, prconde, prcondv,
+                             F77_DBLE_CMPLX_ARG (&dummy_work), lwork, prwork,
+                             info
                              F77_CHAR_ARG_LEN (1)
                              F77_CHAR_ARG_LEN (1)
                              F77_CHAR_ARG_LEN (1)
@@ -289,7 +297,7 @@ EIG::init (const ComplexMatrix& a, bool calc_rev, bool calc_lev, bool balance)
   if (info != 0)
     (*current_liboctave_error_handler) ("zgeevx workspace query failed");
 
-  lwork = static_cast<octave_idx_type> (dummy_work.real ());
+  lwork = static_cast<F77_INT> (dummy_work.real ());
   Array<Complex> work (dim_vector (lwork, 1));
   Complex *pwork = work.fortran_vec ();
 
@@ -298,9 +306,9 @@ EIG::init (const ComplexMatrix& a, bool calc_rev, bool calc_lev, bool balance)
                              F77_CONST_CHAR_ARG2 (calc_rev ? "V" : "N", 1),
                              F77_CONST_CHAR_ARG2 ("N", 1),
                              n, F77_DBLE_CMPLX_ARG (tmp_data), n,
-                             F77_DBLE_CMPLX_ARG (pw), F77_DBLE_CMPLX_ARG (pvl), n,
-                             F77_DBLE_CMPLX_ARG (pvr), n, ilo, ihi, pscale, abnrm,
-                             prconde, prcondv,
+                             F77_DBLE_CMPLX_ARG (pw), F77_DBLE_CMPLX_ARG (pvl),
+                             n, F77_DBLE_CMPLX_ARG (pvr), n, ilo, ihi,
+                             pscale, abnrm, prconde, prcondv,
                              F77_DBLE_CMPLX_ARG (pwork), lwork, prwork, info
                              F77_CHAR_ARG_LEN (1)
                              F77_CHAR_ARG_LEN (1)
@@ -313,9 +321,9 @@ EIG::init (const ComplexMatrix& a, bool calc_rev, bool calc_lev, bool balance)
   if (info > 0)
     (*current_liboctave_error_handler) ("zgeevx failed to converge");
 
-  lambda = wr;
-  v = vrtmp;
-  w = vltmp;
+  m_lambda = wr;
+  m_v = vrtmp;
+  m_w = vltmp;
 
   return info;
 }
@@ -323,12 +331,13 @@ EIG::init (const ComplexMatrix& a, bool calc_rev, bool calc_lev, bool balance)
 octave_idx_type
 EIG::hermitian_init (const ComplexMatrix& a, bool calc_rev, bool calc_lev)
 {
-  octave_idx_type n = a.rows ();
+  F77_INT n = octave::to_f77_int (a.rows ());
+  F77_INT a_nc = octave::to_f77_int (a.cols ());
 
-  if (n != a.cols ())
+  if (n != a_nc)
     (*current_liboctave_error_handler) ("EIG requires square matrix");
 
-  octave_idx_type info = 0;
+  F77_INT info = 0;
 
   ComplexMatrix atmp = a;
   Complex *tmp_data = atmp.fortran_vec ();
@@ -336,10 +345,10 @@ EIG::hermitian_init (const ComplexMatrix& a, bool calc_rev, bool calc_lev)
   ColumnVector wr (n);
   double *pwr = wr.fortran_vec ();
 
-  octave_idx_type lwork = -1;
+  F77_INT lwork = -1;
   Complex dummy_work;
 
-  octave_idx_type lrwork = 3*n;
+  F77_INT lrwork = 3*n;
   Array<double> rwork (dim_vector (lrwork, 1));
   double *prwork = rwork.fortran_vec ();
 
@@ -354,7 +363,7 @@ EIG::hermitian_init (const ComplexMatrix& a, bool calc_rev, bool calc_lev)
   if (info != 0)
     (*current_liboctave_error_handler) ("zheev workspace query failed");
 
-  lwork = static_cast<octave_idx_type> (dummy_work.real ());
+  lwork = static_cast<F77_INT> (dummy_work.real ());
   Array<Complex> work (dim_vector (lwork, 1));
   Complex *pwork = work.fortran_vec ();
 
@@ -371,9 +380,9 @@ EIG::hermitian_init (const ComplexMatrix& a, bool calc_rev, bool calc_lev)
   if (info > 0)
     (*current_liboctave_error_handler) ("zheev failed to converge");
 
-  lambda = ComplexColumnVector (wr);
-  v = calc_rev ? ComplexMatrix (atmp) : ComplexMatrix ();
-  w = calc_lev ? ComplexMatrix (atmp) : ComplexMatrix ();
+  m_lambda = ComplexColumnVector (wr);
+  m_v = (calc_rev ? ComplexMatrix (atmp) : ComplexMatrix ());
+  m_w = (calc_lev ? ComplexMatrix (atmp) : ComplexMatrix ());
 
   return info;
 }
@@ -386,16 +395,19 @@ EIG::init (const Matrix& a, const Matrix& b, bool calc_rev, bool calc_lev,
     (*current_liboctave_error_handler)
       ("EIG: matrix contains Inf or NaN values");
 
-  octave_idx_type n = a.rows ();
-  octave_idx_type nb = b.rows ();
+  F77_INT n = octave::to_f77_int (a.rows ());
+  F77_INT nb = octave::to_f77_int (b.rows ());
 
-  if (n != a.cols () || nb != b.cols ())
+  F77_INT a_nc = octave::to_f77_int (a.cols ());
+  F77_INT b_nc = octave::to_f77_int (b.cols ());
+
+  if (n != a_nc || nb != b_nc)
     (*current_liboctave_error_handler) ("EIG requires square matrix");
 
   if (n != nb)
     (*current_liboctave_error_handler) ("EIG requires same size matrices");
 
-  octave_idx_type info = 0;
+  F77_INT info = 0;
 
   Matrix tmp = b;
   double *tmp_data = tmp.fortran_vec ();
@@ -407,7 +419,7 @@ EIG::init (const Matrix& a, const Matrix& b, bool calc_rev, bool calc_lev,
                                  info
                                  F77_CHAR_ARG_LEN (1)));
 
-      if (a.is_symmetric () && b.is_symmetric () && info == 0)
+      if (a.issymmetric () && b.issymmetric () && info == 0)
         return symmetric_init (a, b, calc_rev, calc_lev);
     }
 
@@ -426,15 +438,15 @@ EIG::init (const Matrix& a, const Matrix& b, bool calc_rev, bool calc_lev,
   Array<double> beta (dim_vector (n, 1));
   double *pbeta = beta.fortran_vec ();
 
-  octave_idx_type tnvr = calc_rev ? n : 0;
+  F77_INT tnvr = (calc_rev ? n : 0);
   Matrix vr (tnvr, tnvr);
   double *pvr = vr.fortran_vec ();
 
-  octave_idx_type tnvl = calc_lev ? n : 0;
+  F77_INT tnvl = (calc_lev ? n : 0);
   Matrix vl (tnvl, tnvl);
   double *pvl = vl.fortran_vec ();
 
-  octave_idx_type lwork = -1;
+  F77_INT lwork = -1;
   double dummy_work;
 
   F77_XFCN (dggev, DGGEV, (F77_CONST_CHAR_ARG2 (calc_lev ? "V" : "N", 1),
@@ -449,7 +461,7 @@ EIG::init (const Matrix& a, const Matrix& b, bool calc_rev, bool calc_lev,
   if (info != 0)
     (*current_liboctave_error_handler) ("dggev workspace query failed");
 
-  lwork = static_cast<octave_idx_type> (dummy_work);
+  lwork = static_cast<F77_INT> (dummy_work);
   Array<double> work (dim_vector (lwork, 1));
   double *pwork = work.fortran_vec ();
 
@@ -468,46 +480,46 @@ EIG::init (const Matrix& a, const Matrix& b, bool calc_rev, bool calc_lev,
   if (info > 0)
     (*current_liboctave_error_handler) ("dggev failed to converge");
 
-  lambda.resize (n);
-  octave_idx_type nvr = calc_rev ? n : 0;
-  v.resize (nvr, nvr);
+  m_lambda.resize (n);
+  F77_INT nvr = (calc_rev ? n : 0);
+  m_v.resize (nvr, nvr);
 
-  octave_idx_type nvl = calc_lev ? n : 0;
-  w.resize (nvl, nvl);
+  F77_INT nvl = (calc_lev ? n : 0);
+  m_w.resize (nvl, nvl);
 
-  for (octave_idx_type j = 0; j < n; j++)
+  for (F77_INT j = 0; j < n; j++)
     {
       if (ai.elem (j) == 0.0)
         {
-          lambda.elem (j) = Complex (ar.elem (j) / beta.elem (j));
-          for (octave_idx_type i = 0; i < nvr; i++)
-            v.elem (i, j) = vr.elem (i, j);
-          for (octave_idx_type i = 0; i < nvl; i++)
-            w.elem (i, j) = vl.elem (i, j);
+          m_lambda.elem (j) = Complex (ar.elem (j) / beta.elem (j));
+          for (F77_INT i = 0; i < nvr; i++)
+            m_v.elem (i, j) = vr.elem (i, j);
+          for (F77_INT i = 0; i < nvl; i++)
+            m_w.elem (i, j) = vl.elem (i, j);
         }
       else
         {
           if (j+1 >= n)
             (*current_liboctave_error_handler) ("EIG: internal error");
 
-          lambda.elem (j) = Complex (ar.elem (j) / beta.elem (j),
-                                     ai.elem (j) / beta.elem (j));
-          lambda.elem (j+1) = Complex (ar.elem (j+1) / beta.elem (j+1),
-                                       ai.elem (j+1) / beta.elem (j+1));
+          m_lambda.elem (j) = Complex (ar.elem (j) / beta.elem (j),
+                                       ai.elem (j) / beta.elem (j));
+          m_lambda.elem (j+1) = Complex (ar.elem (j+1) / beta.elem (j+1),
+                                         ai.elem (j+1) / beta.elem (j+1));
 
-          for (octave_idx_type i = 0; i < nvr; i++)
+          for (F77_INT i = 0; i < nvr; i++)
             {
               double real_part = vr.elem (i, j);
               double imag_part = vr.elem (i, j+1);
-              v.elem (i, j) = Complex (real_part, imag_part);
-              v.elem (i, j+1) = Complex (real_part, -imag_part);
+              m_v.elem (i, j) = Complex (real_part, imag_part);
+              m_v.elem (i, j+1) = Complex (real_part, -imag_part);
             }
-          for (octave_idx_type i = 0; i < nvl; i++)
+          for (F77_INT i = 0; i < nvl; i++)
             {
               double real_part = vl.elem (i, j);
               double imag_part = vl.elem (i, j+1);
-              w.elem (i, j) = Complex (real_part, imag_part);
-              w.elem (i, j+1) = Complex (real_part, -imag_part);
+              m_w.elem (i, j) = Complex (real_part, imag_part);
+              m_w.elem (i, j+1) = Complex (real_part, -imag_part);
             }
           j++;
         }
@@ -520,16 +532,19 @@ octave_idx_type
 EIG::symmetric_init (const Matrix& a, const Matrix& b, bool calc_rev,
                      bool calc_lev)
 {
-  octave_idx_type n = a.rows ();
-  octave_idx_type nb = b.rows ();
+  F77_INT n = octave::to_f77_int (a.rows ());
+  F77_INT nb = octave::to_f77_int (b.rows ());
 
-  if (n != a.cols () || nb != b.cols ())
+  F77_INT a_nc = octave::to_f77_int (a.cols ());
+  F77_INT b_nc = octave::to_f77_int (b.cols ());
+
+  if (n != a_nc || nb != b_nc)
     (*current_liboctave_error_handler) ("EIG requires square matrix");
 
   if (n != nb)
     (*current_liboctave_error_handler) ("EIG requires same size matrices");
 
-  octave_idx_type info = 0;
+  F77_INT info = 0;
 
   Matrix atmp = a;
   double *atmp_data = atmp.fortran_vec ();
@@ -540,7 +555,7 @@ EIG::symmetric_init (const Matrix& a, const Matrix& b, bool calc_rev,
   ColumnVector wr (n);
   double *pwr = wr.fortran_vec ();
 
-  octave_idx_type lwork = -1;
+  F77_INT lwork = -1;
   double dummy_work;
 
   F77_XFCN (dsygv, DSYGV, (1, F77_CONST_CHAR_ARG2 (calc_rev ? "V" : "N", 1),
@@ -554,7 +569,7 @@ EIG::symmetric_init (const Matrix& a, const Matrix& b, bool calc_rev,
   if (info != 0)
     (*current_liboctave_error_handler) ("dsygv workspace query failed");
 
-  lwork = static_cast<octave_idx_type> (dummy_work);
+  lwork = static_cast<F77_INT> (dummy_work);
   Array<double> work (dim_vector (lwork, 1));
   double *pwork = work.fortran_vec ();
 
@@ -572,9 +587,9 @@ EIG::symmetric_init (const Matrix& a, const Matrix& b, bool calc_rev,
   if (info > 0)
     (*current_liboctave_error_handler) ("dsygv failed to converge");
 
-  lambda = ComplexColumnVector (wr);
-  v = calc_rev ? ComplexMatrix (atmp) : ComplexMatrix ();
-  w = calc_lev ? ComplexMatrix (atmp) : ComplexMatrix ();
+  m_lambda = ComplexColumnVector (wr);
+  m_v = (calc_rev ? ComplexMatrix (atmp) : ComplexMatrix ());
+  m_w = (calc_lev ? ComplexMatrix (atmp) : ComplexMatrix ());
 
   return info;
 }
@@ -587,19 +602,22 @@ EIG::init (const ComplexMatrix& a, const ComplexMatrix& b, bool calc_rev,
     (*current_liboctave_error_handler)
       ("EIG: matrix contains Inf or NaN values");
 
-  octave_idx_type n = a.rows ();
-  octave_idx_type nb = b.rows ();
+  F77_INT n = octave::to_f77_int (a.rows ());
+  F77_INT nb = octave::to_f77_int (b.rows ());
 
-  if (n != a.cols () || nb != b.cols ())
+  F77_INT a_nc = octave::to_f77_int (a.cols ());
+  F77_INT b_nc = octave::to_f77_int (b.cols ());
+
+  if (n != a_nc || nb != b_nc)
     (*current_liboctave_error_handler) ("EIG requires square matrix");
 
   if (n != nb)
     (*current_liboctave_error_handler) ("EIG requires same size matrices");
 
-  octave_idx_type info = 0;
+  F77_INT info = 0;
 
   ComplexMatrix tmp = b;
-  Complex*tmp_data = tmp.fortran_vec ();
+  Complex *tmp_data = tmp.fortran_vec ();
 
   if (! force_qz)
     {
@@ -608,7 +626,7 @@ EIG::init (const ComplexMatrix& a, const ComplexMatrix& b, bool calc_rev,
                                  info
                                  F77_CHAR_ARG_LEN (1)));
 
-      if (a.is_hermitian () && b.is_hermitian () && info == 0)
+      if (a.ishermitian () && b.ishermitian () && info == 0)
         return hermitian_init (a, b, calc_rev, calc_lev);
     }
 
@@ -624,22 +642,22 @@ EIG::init (const ComplexMatrix& a, const ComplexMatrix& b, bool calc_rev,
   ComplexColumnVector beta (n);
   Complex *pbeta = beta.fortran_vec ();
 
-  octave_idx_type nvr = calc_rev ? n : 0;
+  F77_INT nvr = (calc_rev ? n : 0);
   ComplexMatrix vrtmp (nvr, nvr);
   Complex *pvr = vrtmp.fortran_vec ();
 
-  octave_idx_type nvl = calc_lev ? n : 0;
+  F77_INT nvl = (calc_lev ? n : 0);
   ComplexMatrix vltmp (nvl, nvl);
   Complex *pvl = vltmp.fortran_vec ();
 
-  octave_idx_type lwork = -1;
+  F77_INT lwork = -1;
   Complex dummy_work;
 
-  octave_idx_type lrwork = 8*n;
+  F77_INT lrwork = 8*n;
   Array<double> rwork (dim_vector (lrwork, 1));
   double *prwork = rwork.fortran_vec ();
 
-  F77_XFCN (zggev, ZGGEV, (F77_CONST_CHAR_ARG2 ("N", 1),
+  F77_XFCN (zggev, ZGGEV, (F77_CONST_CHAR_ARG2 (calc_lev ? "V" : "N", 1),
                            F77_CONST_CHAR_ARG2 (calc_rev ? "V" : "N", 1),
                            n, F77_DBLE_CMPLX_ARG (atmp_data), n,
                            F77_DBLE_CMPLX_ARG (btmp_data), n,
@@ -647,18 +665,19 @@ EIG::init (const ComplexMatrix& a, const ComplexMatrix& b, bool calc_rev,
                            F77_DBLE_CMPLX_ARG (pbeta),
                            F77_DBLE_CMPLX_ARG (pvl), n,
                            F77_DBLE_CMPLX_ARG (pvr), n,
-                           F77_DBLE_CMPLX_ARG (&dummy_work), lwork, prwork, info
+                           F77_DBLE_CMPLX_ARG (&dummy_work), lwork, prwork,
+                           info
                            F77_CHAR_ARG_LEN (1)
                            F77_CHAR_ARG_LEN (1)));
 
   if (info != 0)
     (*current_liboctave_error_handler) ("zggev workspace query failed");
 
-  lwork = static_cast<octave_idx_type> (dummy_work.real ());
+  lwork = static_cast<F77_INT> (dummy_work.real ());
   Array<Complex> work (dim_vector (lwork, 1));
   Complex *pwork = work.fortran_vec ();
 
-  F77_XFCN (zggev, ZGGEV, (F77_CONST_CHAR_ARG2 ("N", 1),
+  F77_XFCN (zggev, ZGGEV, (F77_CONST_CHAR_ARG2 (calc_lev ? "V" : "N", 1),
                            F77_CONST_CHAR_ARG2 (calc_rev ? "V" : "N", 1),
                            n,  F77_DBLE_CMPLX_ARG (atmp_data), n,
                            F77_DBLE_CMPLX_ARG (btmp_data), n,
@@ -676,13 +695,13 @@ EIG::init (const ComplexMatrix& a, const ComplexMatrix& b, bool calc_rev,
   if (info > 0)
     (*current_liboctave_error_handler) ("zggev failed to converge");
 
-  lambda.resize (n);
+  m_lambda.resize (n);
 
-  for (octave_idx_type j = 0; j < n; j++)
-    lambda.elem (j) = alpha.elem (j) / beta.elem (j);
+  for (F77_INT j = 0; j < n; j++)
+    m_lambda.elem (j) = alpha.elem (j) / beta.elem (j);
 
-  v = vrtmp;
-  w = vltmp;
+  m_v = vrtmp;
+  m_w = vltmp;
 
   return info;
 }
@@ -691,16 +710,19 @@ octave_idx_type
 EIG::hermitian_init (const ComplexMatrix& a, const ComplexMatrix& b,
                      bool calc_rev, bool calc_lev)
 {
-  octave_idx_type n = a.rows ();
-  octave_idx_type nb = b.rows ();
+  F77_INT n = octave::to_f77_int (a.rows ());
+  F77_INT nb = octave::to_f77_int (b.rows ());
 
-  if (n != a.cols () || nb != b.cols ())
+  F77_INT a_nc = octave::to_f77_int (a.cols ());
+  F77_INT b_nc = octave::to_f77_int (b.cols ());
+
+  if (n != a_nc || nb != b_nc)
     (*current_liboctave_error_handler) ("EIG requires square matrix");
 
   if (n != nb)
     (*current_liboctave_error_handler) ("EIG requires same size matrices");
 
-  octave_idx_type info = 0;
+  F77_INT info = 0;
 
   ComplexMatrix atmp = a;
   Complex *atmp_data = atmp.fortran_vec ();
@@ -711,10 +733,10 @@ EIG::hermitian_init (const ComplexMatrix& a, const ComplexMatrix& b,
   ColumnVector wr (n);
   double *pwr = wr.fortran_vec ();
 
-  octave_idx_type lwork = -1;
+  F77_INT lwork = -1;
   Complex dummy_work;
 
-  octave_idx_type lrwork = 3*n;
+  F77_INT lrwork = 3*n;
   Array<double> rwork (dim_vector (lrwork, 1));
   double *prwork = rwork.fortran_vec ();
 
@@ -730,7 +752,7 @@ EIG::hermitian_init (const ComplexMatrix& a, const ComplexMatrix& b,
   if (info != 0)
     (*current_liboctave_error_handler) ("zhegv workspace query failed");
 
-  lwork = static_cast<octave_idx_type> (dummy_work.real ());
+  lwork = static_cast<F77_INT> (dummy_work.real ());
   Array<Complex> work (dim_vector (lwork, 1));
   Complex *pwork = work.fortran_vec ();
 
@@ -748,10 +770,9 @@ EIG::hermitian_init (const ComplexMatrix& a, const ComplexMatrix& b,
   if (info > 0)
     (*current_liboctave_error_handler) ("zhegv failed to converge");
 
-  lambda = ComplexColumnVector (wr);
-  v = calc_rev ? ComplexMatrix (atmp) : ComplexMatrix ();
-  w = calc_lev ? ComplexMatrix (atmp) : ComplexMatrix ();
+  m_lambda = ComplexColumnVector (wr);
+  m_v = (calc_rev ? ComplexMatrix (atmp) : ComplexMatrix ());
+  m_w = (calc_lev ? ComplexMatrix (atmp) : ComplexMatrix ());
 
   return info;
 }
-
