@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1996-2021 The Octave Project Developers
+// Copyright (C) 1996-2023 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -47,6 +47,14 @@ static float lo_float_na;
 static int lo_ieee_hw;
 static int lo_ieee_lw;
 
+#if defined (HAVE_MIPS_NAN)
+  #define LO_IEEE_NA_HW 0x7FF040F4
+#else
+  #define LO_IEEE_NA_HW 0x7FF840F4
+#endif
+#define LO_IEEE_NA_LW 0x40000000
+#define LO_IEEE_NA_FLOAT 0x7FC207A2
+
 int
 __lo_ieee_is_NA (double x)
 {
@@ -56,24 +64,6 @@ __lo_ieee_is_NA (double x)
           && t.word[lo_ieee_lw] == LO_IEEE_NA_LW) ? 1 : 0;
 }
 
-int
-__lo_ieee_is_old_NA (double x)
-{
-  lo_ieee_double t;
-  t.value = x;
-  return (__lo_ieee_isnan (x) && t.word[lo_ieee_lw] == LO_IEEE_NA_LW_OLD
-          && t.word[lo_ieee_hw] == LO_IEEE_NA_HW_OLD) ? 1 : 0;
-}
-
-double
-__lo_ieee_replace_old_NA (double x)
-{
-  if (__lo_ieee_is_old_NA (x))
-    return lo_ieee_na_value ();
-  else
-    return x;
-}
-
 double
 lo_ieee_inf_value (void)
 {
