@@ -1,24 +1,31 @@
-## Copyright (C) 1996-2017 John W. Eaton
+########################################################################
+##
+## Copyright (C) 1996-2023 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
-## @deftypefn  {} {} polyint (@var{p})
-## @deftypefnx {} {} polyint (@var{p}, @var{k})
+## @deftypefn  {} {@var{q} =} polyint (@var{p})
+## @deftypefnx {} {@var{q} =} polyint (@var{p}, @var{k})
 ## Return the coefficients of the integral of the polynomial whose
 ## coefficients are represented by the vector @var{p}.
 ##
@@ -27,13 +34,9 @@
 ## @seealso{polyder, polyval}
 ## @end deftypefn
 
-## Author: Tony Richardson <arichard@stark.cc.oh.us>
-## Created: June 1994
-## Adapted-By: jwe
+function q = polyint (p, k)
 
-function retval = polyint (p, k)
-
-  if (nargin < 1 || nargin > 2)
+  if (nargin < 1)
     print_usage ();
   endif
 
@@ -50,7 +53,7 @@ function retval = polyint (p, k)
   lp = length (p);
 
   if (lp == 0)
-    retval = [];
+    q = [];
     return;
   endif
 
@@ -59,7 +62,7 @@ function retval = polyint (p, k)
     p = p.';
   endif
 
-  retval = [(p ./ [lp:-1:1]), k];
+  q = [(p ./ [lp:-1:1]), k];
 
 endfunction
 
@@ -76,5 +79,5 @@ endfunction
 %! B = [length(A):-1:1];
 %! assert (polyint (A), [1./B, 0]);
 
-%!error polyint ()
+%!error <Invalid call> polyint ()
 %!error polyint (ones (2,2))
