@@ -1,4 +1,9 @@
-## Copyright (C) 2000-2019 Paul Kienzle
+########################################################################
+##
+## Copyright (C) 2000-2023 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {[@var{tok}, @var{rem}] =} strtok (@var{str})
@@ -51,12 +58,12 @@
 
 function [tok, rem] = strtok (str, delim)
 
-  if (nargin < 1 || nargin > 2)
+  if (nargin < 1)
     print_usage ();
   elseif (! (ischar (str) || iscellstr (str)))
-    error ("strtok: STR must be a string or cell array of strings.");
+    error ("strtok: STR must be a string or cell array of strings");
   elseif (ischar (str) && ! isvector (str) &&! isempty (str))
-    error ("strtok: STR cannot be a 2-D character array.");
+    error ("strtok: STR cannot be a 2-D character array");
   endif
 
   if (nargin < 2 || isempty (delim))
@@ -130,7 +137,7 @@ function [tok, rem] = strtok (str, delim)
     if (isargout (2))
       rem = cell (size (str));
       rem(eidx) = {""};
-      rem(midx) = tmp(2:2:end);
+      rem (midx) = tmp(2:2:end);
     endif
   endif
 
@@ -194,7 +201,7 @@ endfunction
 %! assert (t{7}, "");
 %! assert (r{7}, "");
 
-## Simple check for 2, 3, and 4 delimeters
+## Simple check for 2, 3, and 4 delimiters
 %!assert (strtok ("this is", "i "), "th")
 %!assert (strtok ("this is", "ij "), "th")
 %!assert (strtok ("this is", "ijk "), "th")
@@ -222,7 +229,6 @@ endfunction
 %! endfor
 
 ## Test input validation
-%!error strtok ()
-%!error strtok ("a", "b", "c")
+%!error <Invalid call> strtok ()
 %!error <STR must be a string> strtok (1, "b")
 %!error <STR cannot be a 2-D> strtok (char ("hello", "world"), "l")
